#!/usr/bin/env python
import re
import time
from tools.multiclass_shared import prepare_data

# run with toy data
[traindat, label_traindat, testdat, label_testdat] = prepare_data()
# run with opt-digits if available
#[traindat, label_traindat, testdat, label_testdat] = prepare_data(False)

parameter_list = [[traindat,testdat,label_traindat,label_testdat,2.1,1,1e-5]]

def classifier_multiclass_ecoc (fm_train_real=traindat,fm_test_real=testdat,label_train_multiclass=label_traindat,label_test_multiclass=label_testdat,lawidth=2.1,C=1,epsilon=1e-5):

	import shogun.Classifier as Classifier
	from modshogun import ECOCStrategy, LibLinear, L2R_L2LOSS_SVC, LinearMulticlassMachine
	from modshogun import MulticlassAccuracy
	from modshogun import RealFeatures, MulticlassLabels

	def nonabstract_class(name):
		try:
		    getattr(Classifier, name)()
		except TypeError:
		    return False
		return True

	encoders = [x for x in dir(Classifier)
		    if re.match(r'ECOC.+Encoder', x) and nonabstract_class(x)]
	decoders = [x for x in dir(Classifier)
		    if re.match(r'ECOC.+Decoder', x) and nonabstract_class(x)]

	fea_train = RealFeatures(fm_train_real)
	fea_test  = RealFeatures(fm_test_real)
	gnd_train = MulticlassLabels(label_train_multiclass)
	if label_test_multiclass is None:
		gnd_test = None
	else:
		gnd_test = MulticlassLabels(label_test_multiclass)

	base_classifier = LibLinear(L2R_L2LOSS_SVC)
	base_classifier.set_bias_enabled(True)

	#print('Testing with %d encoders and %d decoders' % (len(encoders), len(decoders)))
	#print('-' * 70)
	#format_str = '%%15s + %%-10s  %%-10%s %%-10%s %%-10%s'
	#print((format_str % ('s', 's', 's')) % ('encoder', 'decoder', 'codelen', 'time', 'accuracy'))

	def run_ecoc(ier, idr):
		encoder = getattr(Classifier, encoders[ier])()
		decoder = getattr(Classifier, decoders[idr])()

		# whether encoder is data dependent
		if hasattr(encoder, 'set_labels'):
		    encoder.set_labels(gnd_train)
		    encoder.set_features(fea_train)

		strategy = ECOCStrategy(encoder, decoder)
		classifier = LinearMulticlassMachine(strategy, fea_train, base_classifier, gnd_train)
		classifier.train()
		label_pred = classifier.apply(fea_test)
		if gnd_test is not None:
		    evaluator = MulticlassAccuracy()
		    acc = evaluator.evaluate(label_pred, gnd_test)
		else:
		    acc = None

		return (classifier.get_num_machines(), acc)


	for ier in range(len(encoders)):
		for idr in range(len(decoders)):
		    t_begin = time.clock()
		    (codelen, acc) = run_ecoc(ier, idr)
		    if acc is None:
		        acc_fmt = 's'
		        acc = 'N/A'
		    else:
		        acc_fmt = '.4f'

		    t_elapse = time.clock() - t_begin
		    #print((format_str % ('d', '.3f', acc_fmt)) %
		    #        (encoders[ier][4:-7], decoders[idr][4:-7], codelen, t_elapse, acc))

if __name__=='__main__':
    print('MulticlassECOC')
    classifier_multiclass_ecoc(*parameter_list[0])




#!/usr/bin/env python
traindat = '../data/fm_train_real.dat'

parameter_list = [[traindat,3],[traindat,4]]

def clustering_hierarchical_modular (fm_train=traindat,merges=3):
	from modshogun import EuclideanDistance, RealFeatures, Hierarchical, CSVFile

	feats_train=RealFeatures(CSVFile(fm_train))
	distance=EuclideanDistance(feats_train, feats_train)

	hierarchical=Hierarchical(merges, distance)
	hierarchical.train()

	out_distance = hierarchical.get_merge_distances()
	out_cluster = hierarchical.get_cluster_pairs()

	return hierarchical,out_distance,out_cluster

if __name__=='__main__':
	print('Hierarchical')
	clustering_hierarchical_modular(*parameter_list[0])



"""
Test Preprocessor
"""

from modshogun import *

import util


########################################################################
# kernel computation
########################################################################

def _evaluate (indata):
	prefix='kernel_'
	feats=util.get_features(indata, prefix)
	kfun=eval(indata[prefix+'name']+'Kernel')
	kargs=util.get_args(indata, prefix)

	prefix='preprocessor_'
	pargs=util.get_args(indata, prefix)
	feats=util.add_preprocessor(indata[prefix+'name'], feats, *pargs)

	prefix='kernel_'
	kernel=kfun(feats['train'], feats['train'], *kargs)
	km_train=max(abs(
		indata[prefix+'matrix_train']-kernel.get_kernel_matrix()).flat)
	kernel.init(feats['train'], feats['test'])
	km_test=max(abs(
		indata[prefix+'matrix_test']-kernel.get_kernel_matrix()).flat)

	return util.check_accuracy(
		indata[prefix+'accuracy'], km_train=km_train, km_test=km_test)


########################################################################
# public
########################################################################

def test (indata):
	return _evaluate(indata)




#!/usr/bin/env python
data = '../data/fm_train_real.dat'
parameter_list = [[data]]

def converter_multidimensionalscaling_modular (data_fname):
	try:
		import numpy
		from modshogun import RealFeatures, MultidimensionalScaling, EuclideanDistance, CSVFile

		features = RealFeatures(CSVFile(data_fname))

		distance_before = EuclideanDistance()
		distance_before.init(features,features)

		converter = MultidimensionalScaling()
		converter.set_target_dim(2)
		converter.set_landmark(False)
		embedding = converter.apply(features)

		distance_after = EuclideanDistance()
		distance_after.init(embedding,embedding)

		distance_matrix_after = distance_after.get_distance_matrix()
		distance_matrix_before = distance_before.get_distance_matrix()

		return numpy.linalg.norm(distance_matrix_after-distance_matrix_before)/numpy.linalg.norm(distance_matrix_before) < 1e-6
	except ImportError:
		print('No Eigen3 available')

if __name__=='__main__':
	print('MultidimensionalScaling')
	converter_multidimensionalscaling_modular(*parameter_list[0])



"""
Test Distribution
"""
from numpy import inf, nan
from sg import sg
import util

def _evaluate (indata):
	prefix='distribution_'
	# what is sg('likelihood')?
	likelihood=abs(sg('hmm_likelihood')-indata[prefix+'likelihood'])
	return util.check_accuracy(indata[prefix+'accuracy'],
		likelihood=likelihood)

	# best path? which? no_b_trans? trans? trans_deriv?
	if indata['name']=='HMM':
		best_path=0
		best_path_state=0
		for i in xrange(indata[prefix+'num_examples']):
			best_path+=distribution.best_path(i)
			for j in xrange(indata[prefix+'N']):
				best_path_state+=distribution.get_best_path_state(i, j)

		best_path=abs(best_path-indata[prefix+'best_path'])
		best_path_state=abs(best_path_state-\
			indata[prefix+'best_path_state'])

		return util.check_accuracy(indata[prefix+'accuracy'],
			derivatives=derivatives, likelihood=likelihood,
			best_path=best_path, best_path_state=best_path_state)
	else:
		return util.check_accuracy(indata[prefix+'accuracy'],
			derivatives=derivatives, likelihood=likelihood)


def _set_distribution (indata):
	prefix='distribution_'
	if indata[prefix+'name']=='HMM':
		sg('new_hmm', indata[prefix+'N'], indata[prefix+'M'])
		sg('bw')
	else:
		raise NotImplementedError, 'Can\'t yet train other distributions than HMM in static interface.'


########################################################################
# public
########################################################################

def test (indata):
	try:
		util.set_features(indata, 'distribution_')
		_set_distribution(indata)
	except NotImplementedError, e:
		print e
		return True

	return _evaluate(indata)




#!/usr/bin/env python
traindat = '../data/fm_train_real.dat'
label_traindat = '../data/label_train_twoclass.dat'
parameter_list = [[traindat,label_traindat,1.7], [traindat,label_traindat,1.6]]

def kernel_auc_modular (train_fname=traindat,label_fname=label_traindat,width=1.7):
	from modshogun import GaussianKernel, AUCKernel, RealFeatures
	from modshogun import BinaryLabels, CSVFile

	feats_train=RealFeatures(CSVFile(train_fname))
	subkernel=GaussianKernel(feats_train, feats_train, width)

	kernel=AUCKernel(0, subkernel)
	kernel.setup_auc_maximization(BinaryLabels(CSVFile(label_fname)))
	km_train=kernel.get_kernel_matrix()
	return kernel

if __name__=='__main__':
	print('AUC')
	kernel_auc_modular(*parameter_list[0])



#!/usr/bin/env python
from numpy import random,array,sin,round#,sqrt
#from pylab import plot, show, legend, fill_between

parameter_list=[[20,100,6,10,0.05,1, 1], [10,30,7,9,0.5,0.5, 2]]

def regression_gaussian_process_modular (n=100,n_test=100, \
		x_range=6,x_range_test=10,noise_var=0.5,width=1, seed=1):

	from modshogun import RealFeatures, RegressionLabels, GaussianKernel, Math
	try:
		from modshogun import GaussianLikelihood, ZeroMean, \
				ExactInferenceMethod, GaussianProcessRegression
	except ImportError:
		print("Eigen3 needed for Gaussian Processes")
		return

	# reproducable results
	random.seed(seed)
	Math.init_random(17)

	# easy regression data: one dimensional noisy sine wave
	X=random.rand(1,n)*x_range

	X_test=array([[float(i)/n_test*x_range_test for i in range(n_test)]])
	Y_test=sin(X_test)
	Y=sin(X)+random.randn(n)*noise_var

	# shogun representation
	labels=RegressionLabels(Y[0])
	feats_train=RealFeatures(X)
	feats_test=RealFeatures(X_test)

	# GP specification
	shogun_width=width*width*2
	kernel=GaussianKernel(10, shogun_width)
	zmean = ZeroMean()
	lik = GaussianLikelihood()
	lik.set_sigma(noise_var)
	inf = ExactInferenceMethod(kernel, feats_train, zmean, labels, lik)

	# train GP
	gp = GaussianProcessRegression(inf)
	gp.train()

	# some things we can do
	alpha = inf.get_alpha()
	diagonal = inf.get_diagonal_vector()
	cholesky = inf.get_cholesky()

	# get mean and variance vectors
	mean = gp.get_mean_vector(feats_test)
	variance = gp.get_variance_vector(feats_test)

	# plot results
	#plot(X[0],Y[0],'x') # training observations
	#plot(X_test[0],Y_test[0],'-') # ground truth of test
	#plot(X_test[0],mean, '-') # mean predictions of test
	#fill_between(X_test[0],mean-1.96*sqrt(variance),mean+1.96*sqrt(variance),color='grey')  # 95% confidence interval
	#legend(["training", "ground truth", "mean predictions"])

	#show()

	return alpha, diagonal, round(variance,12), round(mean,12), cholesky

if __name__=='__main__':
	print('Gaussian Process Regression')
	regression_gaussian_process_modular(*parameter_list[0])



#!/usr/bin/env python
from tools.load import LoadMatrix
from numpy import ushort

lm=LoadMatrix()
traindat = ushort(lm.load_numbers('../data/fm_train_word.dat'))
testdat = ushort(lm.load_numbers('../data/fm_test_word.dat'))

parameter_list=[[traindat,testdat,1.2],[traindat,testdat,1.2]]

def kernel_linear_word_modular (fm_train_word=traindat,fm_test_word=testdat,scale=1.2):

	from modshogun import LinearKernel, AvgDiagKernelNormalizer
	from modshogun import WordFeatures

	feats_train=WordFeatures(fm_train_word)
	feats_test=WordFeatures(fm_test_word)

	kernel=LinearKernel(feats_train, feats_train)
	kernel.set_normalizer(AvgDiagKernelNormalizer(scale))
	kernel.init(feats_train, feats_train)

	km_train=kernel.get_kernel_matrix()
	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return kernel

if __name__=='__main__':
	print('LinearWord')
	kernel_linear_word_modular(*parameter_list[0])



#!/usr/bin/env python
# create string features with a single string
s=10*'A' + 10*'C' + 10*'G' + 10*'T'

parameter_list=[[s]]

def features_string_sliding_window_modular (strings):
	from modshogun import StringCharFeatures, DNA
	from modshogun import DynamicIntArray

	f=StringCharFeatures([strings], DNA)

	# slide a window of length 5 over features
	# (memory efficient, does not copy strings)
	f.obtain_by_sliding_window(5,1)
	#print(f.get_num_vectors())
	#print(f.get_vector_length(0))
	#print(f.get_vector_length(1))
	#print(f.get_features())

	# slide a window of length 4 over features
	# (memory efficient, does not copy strings)
	f.obtain_by_sliding_window(4,1)
	#print(f.get_num_vectors())
	#print(f.get_vector_length(0))
	#print(f.get_vector_length(1))
	#print(f.get_features())

	# extract string-windows at position 0,6,16,25 of window size 4
	# (memory efficient, does not copy strings)
	f.set_features([s])
	positions=DynamicIntArray()
	positions.append_element(0)
	positions.append_element(6)
	positions.append_element(16)
	positions.append_element(25)

	f.obtain_by_position_list(4,positions)
	#print(f.get_features())

	# now extract windows of size 8 from same positon list
	f.obtain_by_position_list(8,positions)
	#print(f.get_features())
	return f

if __name__=='__main__':
	print('Sliding Window')
	features_string_sliding_window_modular(*parameter_list[0])



#!/usr/bin/env python

import numpy as np
import matplotlib.pyplot as plt
import latex_plot_inits

parameter_list = [[20, 5, 1., 1000, 1, None, 5], [100, 5, 1., 1000, 1, None, 10]]

def classifier_perceptron_graphical(n=100, distance=5, learn_rate=1., max_iter=1000, num_threads=1, seed=None, nperceptrons=5):
	from modshogun import RealFeatures, BinaryLabels
	from modshogun import Perceptron
	from modshogun import MSG_INFO

	# 2D data
	_DIM = 2

	# To get the nice message that the perceptron has converged
	dummy = BinaryLabels()
	dummy.io.set_loglevel(MSG_INFO)

	np.random.seed(seed)

	# Produce some (probably) linearly separable training data by hand
	# Two Gaussians at a far enough distance
	X = np.array(np.random.randn(_DIM,n))+distance
	Y = np.array(np.random.randn(_DIM,n))
	label_train_twoclass = np.hstack((np.ones(n), -np.ones(n)))

	fm_train_real = np.hstack((X,Y))
	feats_train = RealFeatures(fm_train_real)
	labels = BinaryLabels(label_train_twoclass)

	perceptron = Perceptron(feats_train, labels)
	perceptron.set_learn_rate(learn_rate)
	perceptron.set_max_iter(max_iter)
	perceptron.set_initialize_hyperplane(False)

	# Find limits for visualization
	x_min = min(np.min(X[0,:]), np.min(Y[0,:]))
	x_max = max(np.max(X[0,:]), np.max(Y[0,:]))

	y_min = min(np.min(X[1,:]), np.min(Y[1,:]))
	y_max = max(np.max(X[1,:]), np.max(Y[1,:]))

	for i in xrange(nperceptrons):
		# Initialize randomly weight vector and bias
		perceptron.set_w(np.random.random(2))
		perceptron.set_bias(np.random.random())

		# Run the perceptron algorithm
		perceptron.train()

		# Construct the hyperplane for visualization
		# Equation of the decision boundary is w^T x + b = 0
		b = perceptron.get_bias()
		w = perceptron.get_w()

		hx = np.linspace(x_min-1,x_max+1)
		hy = -w[1]/w[0] * hx

		plt.plot(hx, -1/w[1]*(w[0]*hx+b))

	# Plot the two-class data
	plt.scatter(X[0,:], X[1,:], s=40, marker='o', facecolors='none', edgecolors='b')
	plt.scatter(Y[0,:], Y[1,:], s=40, marker='s', facecolors='none', edgecolors='r')

	# Customize the plot
	plt.axis([x_min-1, x_max+1, y_min-1, y_max+1])
	plt.title('Rosenblatt\'s Perceptron Algorithm')
	plt.xlabel('x')
	plt.ylabel('y')
	plt.show()

	return perceptron

if __name__=='__main__':
	print('Perceptron graphical')
	classifier_perceptron_graphical(*parameter_list[0])



#!/usr/bin/env python
traindat = '../data/fm_train_real.dat'
testdat = '../data/fm_test_real.dat'

parameter_list=[[traindat,testdat,1.9],[traindat,testdat,1.7]]

def kernel_io_modular (train_fname=traindat,test_fname=testdat,width=1.9):
	from modshogun import RealFeatures, GaussianKernel, CSVFile

	feats_train=RealFeatures(CSVFile(train_fname))
	feats_test=RealFeatures(CSVFile(test_fname))

	kernel=GaussianKernel(feats_train, feats_train, width)
	km_train=kernel.get_kernel_matrix()
	f=CSVFile("gaussian_train.csv","w")
	kernel.save(f)
	del f

	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	f=CSVFile("gaussian_test.csv","w")
	kernel.save(f)
	del f

	#clean up
	import os
	os.unlink("gaussian_test.csv")
	os.unlink("gaussian_train.csv")

	return km_train, km_test, kernel

if __name__=='__main__':
	print('Gaussian')
	kernel_io_modular(*parameter_list[0])



"""
Generator for Kernel

A word about args: it is organised as two correlated tuples, because the
order of the elements of dicts in Python is arbitrary, meaning that the item
added first might be the last when iterating over the dict.
"""

import numpy
import shogun.Kernel as kernel
from shogun.Features import CombinedFeatures, TOPFeatures, FKFeatures, CUBE, RAWBYTE
from shogun.Classifier import PluginEstimate
from shogun.Distance import CanberraMetric
from shogun.Distribution import HMM, LinearHMM, BW_NORMAL
from shogun.Library import Math_init_random

import fileop
import featop
import dataop
import category


##################################################################
## compute funcs
##################################################################

def _compute_pie (feats, params):
	"""Compute a kernel with PluginEstimate.

	@param feats kernel features
	@param params dict containing various kernel parameters
	"""

	output=fileop.get_output(category.KERNEL, params)

	lab, labels=dataop.get_labels(feats['train'].get_num_vectors())
	output['classifier_labels']=lab
	pie=PluginEstimate()
	pie.set_labels(labels)
	pie.set_features(feats['train'])
	pie.train()

	kfun=eval('kernel.'+params['name']+'Kernel')
	kern=kfun(feats['train'], feats['train'], pie)
	output['kernel_matrix_train']=kern.get_kernel_matrix()
	kern.init(feats['train'], feats['test'])
	pie.set_features(feats['test'])
	output['kernel_matrix_test']=kern.get_kernel_matrix()

	classified=pie.apply().get_labels()
	output['classifier_classified']=classified

	fileop.write(category.KERNEL, output)


def _compute_top_fisher (feats, pout):
	"""Compute PolyKernel with TOP or FKFeatures

	@param feats features of the kernel
	@param pout previously gathered data ready to be written to file
	"""

	params={
		'name': 'Poly',
		'accuracy': 1e-6,
		'args': {
			'key': ('size', 'degree', 'inhomogene'),
			'val': (10, 1, False)
		}
	}
	output=fileop.get_output(category.KERNEL, params)
	output.update(pout)

	kfun=eval('kernel.'+params['name']+'Kernel')
	kern=kfun(feats['train'], feats['train'], *params['args']['val'])
	output['kernel_matrix_train']=kern.get_kernel_matrix()
	kern.init(feats['train'], feats['test'])
	output['kernel_matrix_test']=kern.get_kernel_matrix()

	fileop.write(category.KERNEL, output)


def _compute (feats, params, pout=None):
	"""
	Compute a kernel and write gathered data to file.

	@param name name of the kernel
	@param feats features of the kernel
	@param params dict with parameters to kernel
	@param pout previously gathered data ready to be written to file
	"""

	output=fileop.get_output(category.KERNEL, params)
	if pout:
		output.update(pout)

	kfun=eval('kernel.'+params['name']+'Kernel')
	if params.has_key('args'):
		kern=kfun(*params['args']['val'])
	else:
		kern=kfun()

	if params.has_key('normalizer'):
		kern.set_normalizer(params['normalizer'])
	kern.init(feats['train'], feats['train'])

	output['kernel_matrix_train']=kern.get_kernel_matrix()
	kern.init(feats['train'], feats['test'])
	output['kernel_matrix_test']=kern.get_kernel_matrix()

	fileop.write(category.KERNEL, output)


##################################################################
## run funcs
##################################################################

def _run_auc ():
	"""Run AUC kernel."""

	# handle subkernel
	params={
		'name': 'Gaussian',
		'data': dataop.get_rand(),
		'feature_class': 'simple',
		'feature_type': 'Real',
		'args': {'key': ('size', 'width'), 'val': (10, 1.7)}
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'], params['data'])
	subk=kernel.GaussianKernel(*params['args']['val'])
	subk.init(feats['train'], feats['test'])
	output=fileop.get_output(category.KERNEL, params, 'subkernel0_')

	# handle AUC
	params={
		'name': 'AUC',
		'data': dataop.get_rand(numpy.ushort, num_feats=2,
			max_train=dataop.NUM_VEC_TRAIN, max_test=dataop.NUM_VEC_TEST),
		'feature_class': 'simple',
		'feature_type': 'Word',
		'accuracy': 1e-8,
		'args': {'key': ('size', 'subkernel'), 'val': (10, subk)}
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'], params['data'])
	_compute(feats, params, output)


def _run_combined ():
	"""Run Combined kernel."""

	kern=kernel.CombinedKernel()
	feats={'train': CombinedFeatures(), 'test': CombinedFeatures()}
	output={}
	params={
		'name': 'Combined',
		'accuracy': 1e-7
	}
	subkdata=[
		{
			'name': 'FixedDegreeString',
			'feature_class': 'string',
			'feature_type': 'Char',
			'args': {'key': ('size', 'degree'), 'val': (10, 3)}
		},
		{
			'name': 'PolyMatchString',
			'feature_class': 'string',
			'feature_type': 'Char',
			'args': {
				'key': ('size', 'degree', 'inhomogene'),
				'val': (10, 3, True)
			}
		},
		{
			'name': 'LocalAlignmentString',
			'feature_class': 'string',
			'feature_type': 'Char',
			'args': {'key': ('size',), 'val': (10,)}
		}
	]

	i=0
	for sd in subkdata:
		kfun=eval('kernel.'+sd['name']+'Kernel')
		subk=kfun(*sd['args']['val'])
		sd['data']=dataop.get_dna()
		subkfeats=featop.get_features(
			sd['feature_class'], sd['feature_type'], sd['data'])
		output.update(
			fileop.get_output(category.KERNEL, sd, 'subkernel'+str(i)+'_'))

		kern.append_kernel(subk)
		feats['train'].append_feature_obj(subkfeats['train'])
		feats['test'].append_feature_obj(subkfeats['test'])

		i+=1

	output.update(fileop.get_output(category.KERNEL, params))
	kern.init(feats['train'], feats['train'])
	output['kernel_matrix_train']=kern.get_kernel_matrix()
	kern.init(feats['train'], feats['test'])
	output['kernel_matrix_test']=kern.get_kernel_matrix()

	fileop.write(category.KERNEL, output)


def _run_subkernels ():
	"""Run all kernels handling subkernels."""

	_run_auc()
	_run_combined()


def _run_custom ():
	"""Run Custom kernel."""

	params={
		'name': 'Custom',
		'accuracy': 1e-7,
		'feature_class': 'simple',
		'feature_type': 'Real'
	}
	dim_square=7
	data=dataop.get_rand(dim_square=dim_square)
	feats=featop.get_features(
		params['feature_class'], params['feature_type'], data)
	data=data['train']
	symdata=data+data.T

	lowertriangle=numpy.array([symdata[(x,y)] for x in xrange(symdata.shape[1])
		for y in xrange(symdata.shape[0]) if y<=x])
	kern=kernel.CustomKernel()
	#kern.init(feats['train'], feats['train']
	kern.set_triangle_kernel_matrix_from_triangle(lowertriangle)
	km_triangletriangle=kern.get_kernel_matrix()
	kern.set_triangle_kernel_matrix_from_full(symdata)
	km_fulltriangle=kern.get_kernel_matrix()
	kern.set_full_kernel_matrix_from_full(data)
	km_fullfull=kern.get_kernel_matrix()

	output={
		'kernel_matrix_triangletriangle': km_triangletriangle,
		'kernel_matrix_fulltriangle': km_fulltriangle,
		'kernel_matrix_fullfull': km_fullfull,
		'kernel_symdata': numpy.matrix(symdata),
		'kernel_data': numpy.matrix(data),
		'kernel_dim_square': dim_square
	}
	output.update(fileop.get_output(category.KERNEL, params))

	fileop.write(category.KERNEL, output)


def _run_distance ():
	"""Run distance kernel."""

	params={
		'name': 'Distance',
		'accuracy': 1e-9,
		'feature_class': 'simple',
		'feature_type': 'Real',
		'data': dataop.get_rand(),
		'args': {
			'key': ('size', 'width', 'distance'),
			'val': (10, 1.7, CanberraMetric())
		}
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'], params['data'])

	_compute(feats, params)


def _run_feats_byte ():
	"""Run kernel with ByteFeatures."""

	params={
		'name': 'Linear',
		'accuracy': 1e-8,
		'feature_class': 'simple',
		'feature_type': 'Byte',
		'data': dataop.get_rand(dattype=numpy.ubyte),
		'normalizer': kernel.AvgDiagKernelNormalizer()
	}
	feats=featop.get_features(params['feature_class'], params['feature_type'],
		params['data'], RAWBYTE)

	_compute(feats, params)


def _run_mindygram ():
	"""Run Mindygram kernel."""
	return

	params={
		'name': 'MindyGram',
		'accuracy': 1e-8,
		'data': dataop.get_dna(),
		'feature_class': 'mindy',
		'args': {'key': ('measure', 'width'), 'val': ('MEASURE', 1.5)}
	}
	feats={
		'train': MindyGramFeatures('DNA', 'freq', '%20.,', 0),
		'test': MindyGramFeatures('DNA', 'freq', '%20.,', 0)
	}

	_compute(feats, params)


def _run_feats_real ():
	"""Run kernel with RealFeatures."""

	params={
		'data': dataop.get_rand(),
		'accuracy': 1e-8,
		'feature_class': 'simple',
		'feature_type': 'Real'
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'], params['data'])
	sparsefeats=featop.get_features(
		params['feature_class'], params['feature_type'],
		params['data'], sparse=True)

	params['name']='Gaussian'
	params['args']={'key': ('size', 'width',), 'val': (10, 1.3)}
	_compute(feats, params)

	params['name']='GaussianShift'
	params['args']={
		'key': ('size', 'width', 'max_shift', 'shift_step'),
		'val': (10, 1.3, 2, 1)
	}
	_compute(feats, params)

	params['name']='Gaussian'
	params['args']={'key': ('size', 'width'), 'val': (10, 1.7)}
	_compute(sparsefeats, params)

	params['accuracy']=0
	params['name']='Const'
	params['args']={'key': ('c',), 'val': (23.,)}
	_compute(feats, params)

	params['name']='Diag'
	params['args']={'key': ('size', 'diag'), 'val': (10, 23.)}
	_compute(feats, params)

	params['accuracy']=1e-9
	params['name']='Sigmoid'
	params['args']={
		'key': ('size', 'gamma', 'coef0'),
		'val': (10, 1.1, 1.3)
	}
	_compute(feats, params)
	params['args']['val']=(10, 0.5, 0.7)
	_compute(feats, params)

	params['name']='Chi2'
	params['args']={'key': ('size', 'width'), 'val': (10, 1.2)}
	_compute(feats, params)

	params['accuracy']=1e-8
	params['name']='Poly'
	params['args']={
		'key': ('size', 'degree', 'inhomogene'),
		'val': (10, 3, True)
	}
	_compute(sparsefeats, params)
	params['args']['val']=(10, 3, False)
	_compute(sparsefeats, params)

	params['name']='Poly'
	params['normalizer']=kernel.SqrtDiagKernelNormalizer()
	params['args']={
		'key': ('size', 'degree', 'inhomogene'),
		'val': (10, 3, True)
	}
	_compute(feats, params)
	params['args']['val']=(10, 3, False)
	_compute(feats, params)

	params['normalizer']=kernel.AvgDiagKernelNormalizer()
	del params['args']
	params['name']='Linear'
	_compute(feats, params)
	params['name']='Linear'
	_compute(sparsefeats, params)


def _run_feats_string ():
	"""Run kernel with StringFeatures."""

	params = {
		'accuracy': 1e-9,
		'data': dataop.get_dna(),
		'feature_class': 'string',
		'feature_type': 'Char',
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'], params['data'])

	params['name']='FixedDegreeString'
	params['args']={'key': ('size', 'degree'), 'val': (10, 3)}
	_compute(feats, params)

	params['accuracy']=1e-9
	params['name']='LocalAlignmentString'
	params['args']={'key': ('size',), 'val': (10,)}
	_compute(feats, params)

	params['accuracy']=1e-10
	params['name']='PolyMatchString'
	params['args']={
		'key': ('size', 'degree', 'inhomogene'),
		'val': (10, 3, True)
	}
	_compute(feats, params)
	params['args']['val']=(10, 3, False)
	_compute(feats, params)

	params['accuracy']=1e-15
	params['name']='SimpleLocalityImprovedString'
	params['args']={
		'key': ('size', 'length', 'inner_degree', 'outer_degree'),
		'val': (10, 5, 7, 5)
	}
	_compute(feats, params)
	# buggy:
	#params['name']='LocalityImprovedString'
	#_compute(feats, params)

	params['name']='WeightedDegreeString'
	params['accuracy']=1e-9
	params['args']={'key': ('degree',), 'val': (20,)}
	_compute(feats, params)
	params['args']={'key': ('degree',), 'val': (1,)}
	_compute(feats, params)

	params['name']='WeightedDegreePositionString'
	params['args']={'key': ('size', 'degree'), 'val': (10, 20)}
	_compute(feats, params)
	params['args']={'key': ('size', 'degree'), 'val': (10, 1)}
	_compute(feats, params)

	params['name']='OligoString'
	params['args']={'key': ('size', 'k', 'width'), 'val': (10, 3, 1.2)}
	_compute(feats, params)
	params['args']={'key': ('size', 'k', 'width'), 'val': (10, 4, 1.7)}
	_compute(feats, params)

	params['name']='LinearString'
	params['accuracy']=1e-8
	params['normalizer']=kernel.AvgDiagKernelNormalizer()
	del params['args']
	_compute(feats, params)


def _run_feats_word ():
	"""Run kernel with WordFeatures."""

	maxval=42
	params={
		'name': 'Linear',
		'accuracy': 1e-8,
		'feature_class': 'simple',
		'feature_type': 'Word',
		'data': dataop.get_rand(
			dattype=numpy.ushort, max_train=maxval, max_test=maxval),
		'normalizer': kernel.AvgDiagKernelNormalizer()
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'], params['data'])

	_compute(feats, params)


def _run_feats_string_complex ():
	"""Run kernel with complex StringFeatures."""

	params={
		'data': dataop.get_dna(),
		'feature_class': 'string_complex'
	}

	params['feature_type']='Word'
	wordfeats=featop.get_features(
		params['feature_class'], params['feature_type'], params['data'])

	params['name']='CommWordString'
	params['accuracy']=1e-9
	params['args']={'key': ('size', 'use_sign'), 'val': (10, False)}
	_compute(wordfeats, params)
	params['name']='WeightedCommWordString'
	_compute(wordfeats, params)

	params['name']='PolyMatchWordString'
	params['accuracy']=1e-10
	params['args']={
		'key': ('size', 'degree', 'inhomogene'),
		'val': (10, 3, True)
	}
	_compute(wordfeats, params)
	params['args']['val']=(10, 3, False)
	_compute(wordfeats, params)

	params['name']='MatchWordString'
	params['args']={'key': ('size', 'degree'), 'val': (10, 3)}
	_compute(wordfeats, params)

	params['feature_type']='Ulong'
	params['accuracy']=1e-9
	ulongfeats=featop.get_features(
		params['feature_class'], params['feature_type'], params['data'])
	params['name']='CommUlongString'
	params['args']={'key': ('size', 'use_sign'), 'val': (10, False)}
	_compute(ulongfeats, params)


def _run_pie ():
	"""Run kernel with PluginEstimate."""

	params={
		'data': dataop.get_dna(),
		'accuracy': 1e-6,
		'feature_class': 'string_complex',
		'feature_type': 'Word'
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'], params['data'])

	params['name']='HistogramWordString'
	_compute_pie(feats, params)
	params['name']='SalzbergWordString'
	_compute_pie(feats, params)


def _run_top_fisher ():
	"""Run Linear Kernel with {Top,Fisher}Features."""

	# put some constantness into randomness
	Math_init_random(dataop.INIT_RANDOM)

	data=dataop.get_cubes(4, 8)
	prefix='topfk_'
	params={
		prefix+'N': 3,
		prefix+'M': 6,
		prefix+'pseudo': 1e-1,
		prefix+'order': 1,
		prefix+'gap': 0,
		prefix+'reverse': False,
		prefix+'alphabet': 'CUBE',
		prefix+'feature_class': 'string_complex',
		prefix+'feature_type': 'Word',
		prefix+'data_train': numpy.matrix(data['train']),
		prefix+'data_test': numpy.matrix(data['test'])
	}

	wordfeats=featop.get_features(
		params[prefix+'feature_class'], params[prefix+'feature_type'],
		data, eval(params[prefix+'alphabet']),
		params[prefix+'order'], params[prefix+'gap'], params[prefix+'reverse'])
	pos_train=HMM(wordfeats['train'],
		params[prefix+'N'], params[prefix+'M'], params[prefix+'pseudo'])
	pos_train.train()
	pos_train.baum_welch_viterbi_train(BW_NORMAL)
	neg_train=HMM(wordfeats['train'],
		params[prefix+'N'], params[prefix+'M'], params[prefix+'pseudo'])
	neg_train.train()
	neg_train.baum_welch_viterbi_train(BW_NORMAL)
	pos_test=HMM(pos_train)
	pos_test.set_observations(wordfeats['test'])
	neg_test=HMM(neg_train)
	neg_test.set_observations(wordfeats['test'])
	feats={}

	feats['train']=TOPFeatures(10, pos_train, neg_train, False, False)
	feats['test']=TOPFeatures(10, pos_test, neg_test, False, False)
	params[prefix+'name']='TOP'
	_compute_top_fisher(feats, params)

	feats['train']=FKFeatures(10, pos_train, neg_train)
	feats['train'].set_opt_a(-1) #estimate prior
	feats['test']=FKFeatures(10, pos_test, neg_test)
	feats['test'].set_a(feats['train'].get_a()) #use prior from training data
	params[prefix+'name']='FK'
	_compute_top_fisher(feats, params)


def run ():
	"""Run generator for all kernels."""

	#_run_mindygram()
	_run_top_fisher()
	_run_pie()
	_run_custom()
	_run_distance()
	_run_subkernels()

	_run_feats_byte()
	_run_feats_real()
	_run_feats_string()
	_run_feats_string_complex()
	_run_feats_word()



from tools.load import LoadMatrix
from numpy import ushort
from sg import sg
lm=LoadMatrix()

trainword=ushort(lm.load_numbers('../data/fm_test_word.dat'))
testword=ushort(lm.load_numbers('../data/fm_test_word.dat'))
parameter_list=[[trainword,testword,10,1.4],
	       [trainword,testword,11,1.5]]

def kernel_linearword (fm_train_word=trainword,fm_test_word=testword,
		       size_cache=10, scale=1.4):
	sg('set_features', 'TRAIN', fm_train_word)
	sg('set_features', 'TEST', fm_test_word)
	sg('set_kernel', 'LINEAR', 'WORD', size_cache, scale)
	km=sg('get_kernel_matrix', 'TRAIN')
	km=sg('get_kernel_matrix', 'TEST')
	return km

if __name__=='__main__':
	print('LinearWord')
	kernel_linearword(*parameter_list[0])



#!/usr/bin/env python
traindat = '../data/fm_train_real.dat'
testdat = '../data/fm_test_real.dat'

parameter_list=[[traindat,testdat, 1.0],[traindat,testdat, 5.0]]

def kernel_inversemultiquadric_modular (train_fname=traindat,test_fname=testdat, shift_coef=1.0):
	from modshogun import RealFeatures, InverseMultiQuadricKernel, EuclideanDistance, CSVFile

	feats_train=RealFeatures(CSVFile(train_fname))
	feats_test=RealFeatures(CSVFile(test_fname))

	distance=EuclideanDistance(feats_train, feats_train)
	kernel=InverseMultiQuadricKernel(feats_train, feats_train, shift_coef, distance)
	km_train=kernel.get_kernel_matrix()

	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return km_train,km_test,kernel


if __name__=='__main__':
	print('InverseMultiquadric')
	kernel_inversemultiquadric_modular(*parameter_list[0])



#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Written (W) 2006-2007 Soeren Sonnenburg
# Written (W) 2007 Gunnar Raetsch
# Copyright (C) 2007-2008 Fraunhofer Institute FIRST and Max-Planck-Society
#

import numpy

class content_sensors:
	def __init__(self, model):
		self.dict_weights_intron=numpy.array(model.dict_weights_intron, dtype=numpy.float64)
		self.dict_weights_coding=numpy.array(model.dict_weights_coding, dtype=numpy.float64)

		self.dicts=numpy.concatenate((self.dict_weights_coding,self.dict_weights_intron, self.dict_weights_coding, self.dict_weights_intron, self.dict_weights_coding,self.dict_weights_intron, self.dict_weights_coding, self.dict_weights_intron), axis=0)

		self.dicts[0, 64:] = 0      # only order 3 info
		self.dicts[1, 64:] = 0      # only order 3 info
		self.dicts[2, 0:64] = 0     # only order 4 info
		self.dicts[2, 320:] = 0
		self.dicts[3, 0:64] = 0     # only order 4 info
		self.dicts[3, 320:] = 0
		self.dicts[4, 0:320] = 0    # only order 5 info
		self.dicts[4, 1344:] = 0
		self.dicts[5, 0:320] = 0    # only order 5 info
		self.dicts[5, 1344:] = 0
		self.dicts[6, 0:1344] = 0   # only order 6 info
		self.dicts[7, 0:1344] = 0   # only order 6 info

		self.model = model

	def get_dict_weights(self):
		return self.dicts.T

	def initialize_content(self, dyn):
		dyn.init_svm_arrays(len(self.model.word_degree), len(self.model.mod_words))

		word_degree = numpy.array(self.model.word_degree, numpy.int32)
		dyn.init_word_degree_array(word_degree)

		mod_words = numpy.array(4**word_degree, numpy.int32)
		dyn.init_num_words_array(mod_words)

		cum_mod_words=numpy.zeros(len(mod_words)+1, numpy.int32)
		cum_mod_words[1:] = numpy.cumsum(mod_words)
		dyn.init_cum_num_words_array(cum_mod_words)

		dyn.init_mod_words_array(numpy.array(self.model.mod_words, numpy.int32))
		dyn.init_sign_words_array(numpy.array(self.model.sign_words, numpy.bool))
		dyn.init_string_words_array(numpy.zeros(len(self.model.sign_words), numpy.int32))

		assert(dyn.check_svm_arrays())



#!/usr/bin/env python
from tools.load import LoadMatrix

lm=LoadMatrix()
data = lm.load_numbers('../data/fm_train_real.dat')

parameter_list = [[data, 20], [data, 30]]

def preprocessor_dimensionreductionpreprocessor_modular (data, k):
	from modshogun import RealFeatures
	from modshogun import DimensionReductionPreprocessor
	from modshogun import LocallyLinearEmbedding

	features = RealFeatures(data)

	converter = LocallyLinearEmbedding()
	converter.set_k(k)

	preprocessor = DimensionReductionPreprocessor(converter)
	preprocessor.init(features)
	preprocessor.apply_to_feature_matrix(features)

	return features


if __name__=='__main__':
	print('DimensionReductionPreprocessor')
	preprocessor_dimensionreductionpreprocessor_modular(*parameter_list[0])




#!/usr/bin/env python

#############################################################################################
#                                                                                           #
#    This program is free software; you can redistribute it and/or modify                   #
#    it under the terms of the GNU General Public License as published by                   #
#    the Free Software Foundation; either version 3 of the License, or                      #
#    (at your option) any later version.                                                    #
#                                                                                           #
#    This program is distributed in the hope that it will be useful,                        #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of                         #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                           #
#    GNU General Public License for more details.                                           #
#                                                                                           #
#    You should have received a copy of the GNU General Public License                      #
#    along with this program; if not, see http://www.gnu.org/licenses                       #
#    or write to the Free Software Foundation, Inc., 51 Franklin Street,                    #
#    Fifth Floor, Boston, MA 02110-1301  USA                                                #
#                                                                                           #
#############################################################################################

import sys
import random
from esvm.experiment import svm_cv, svm_pred, svm_poim, svm_eval, svm_modelsel

if __name__ == '__main__':

    if len(sys.argv)<2:
        sys.stderr.write("usage: %s [cv|pred|modelsel|eval|poim] parameters\n" % sys.argv[0])
        sys.exit(-1)

    random.seed()

    topmode = sys.argv[1]

    if topmode == 'cv':
        svm_cv(sys.argv)
    elif topmode == 'pred':
        svm_pred(sys.argv)
    elif topmode == 'poim':
        svm_poim(sys.argv)
    elif topmode == 'eval':
        svm_eval(sys.argv)
    elif topmode == 'modelsel':
        svm_modelsel(sys.argv)
    else:
        sys.stderr.write( "unknown mode %s (use: cv, pred, poim, eval)\n" % topmode)
        sys.exit(-1)

    sys.exit(0)




#!/usr/bin/env python
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Written (W) 2012 Heiko Strathmann
# Copyright (C) 2012 Berlin Institute of Technology and Max-Planck-Society
#
traindat = '../data/fm_train_real.dat'
label_traindat = '../data/label_train_twoclass.dat'

parameter_list = [[traindat,label_traindat,0.8,1e-6],[traindat,label_traindat,0.9,1e-7]]

def evaluation_cross_validation_regression (train_fname=traindat,label_fname=label_traindat,width=0.8,tau=1e-6):
	from modshogun import CrossValidation, CrossValidationResult
	from modshogun import MeanSquaredError, CrossValidationSplitting
	from modshogun import RegressionLabels, RealFeatures
	from modshogun import GaussianKernel, KernelRidgeRegression, CSVFile

	# training data
	features=RealFeatures(CSVFile(train_fname))
	labels=RegressionLabels(CSVFile(label_fname))

	# kernel and predictor
	kernel=GaussianKernel()
	predictor=KernelRidgeRegression(tau, kernel, labels)

	# splitting strategy for 5 fold cross-validation (for classification its better
	# to use "StratifiedCrossValidation", but here, the std x-val is used
	splitting_strategy=CrossValidationSplitting(labels, 5)

	# evaluation method
	evaluation_criterium=MeanSquaredError()

	# cross-validation instance
	cross_validation=CrossValidation(predictor, features, labels,
			splitting_strategy, evaluation_criterium)

	# (optional) repeat x-val 10 times
	cross_validation.set_num_runs(10)

	# (optional) request 95% confidence intervals for results (not actually needed
	# for this toy example)
	cross_validation.set_conf_int_alpha(0.05)

	# (optional) tell machine to precompute kernel matrix. speeds up. may not work
	predictor.data_lock(labels, features)

	# perform cross-validation and print(results)
	result=cross_validation.evaluate()
	#print("mean:", result.mean)
	#if result.has_conf_int:
	#    print("[", result.conf_int_low, ",", result.conf_int_up, "] with alpha=", result.conf_int_alpha)

if __name__=='__main__':
	print('Evaluation CrossValidationClassification')
	evaluation_cross_validation_regression(*parameter_list[0])



from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()

traindna=lm.load_dna('../data/fm_train_dna.dat')
testdna=lm.load_dna('../data/fm_test_dna.dat')
trainlabel=lm.load_labels('../data/label_train_dna.dat')
parameter_list=[[traindna,testdna,trainlabel,10,3,0,'n'],
		[traindna,testdna,trainlabel,11,4,0,'n']]

def kernel_pluginestimatehistogram (fm_train_dna=traindna,fm_test_dna=testdna,
				 label_train_dna=trainlabel,size_cache=10,
				 order=3,gap=0,reverse='n',):

	sg('set_features', 'TRAIN', fm_train_dna, 'DNA')
	sg('convert', 'TRAIN', 'STRING', 'CHAR', 'STRING', 'WORD', order, order-1, gap, reverse)

	sg('set_features', 'TEST', fm_test_dna, 'DNA')
	sg('convert', 'TEST', 'STRING', 'CHAR', 'STRING', 'WORD', order, order-1, gap, reverse)

	pseudo_pos=1e-1
	pseudo_neg=1e-1
	sg('new_plugin_estimator', pseudo_pos, pseudo_neg)
	sg('set_labels', 'TRAIN', label_train_dna)
	sg('train_estimator')

	sg('set_kernel', 'HISTOGRAM', 'WORD', size_cache)
	km=sg('get_kernel_matrix', 'TRAIN')

# not supported yet
#	lab=sg('plugin_estimate_classify')
	km=sg('get_kernel_matrix', 'TEST')
	return km

if __name__=='__main__':
	print('PluginEstimate w/ HistogramWord')
	kernel_pluginestimatehistogram(*parameter_list[0])



#!/usr/bin/env python
from tools.load import LoadMatrix
lm=LoadMatrix()

traindat = lm.load_numbers('../data/fm_train_real.dat')
testdat = lm.load_numbers('../data/fm_test_real.dat')

parameter_list = [[traindat,testdat,1.5,10],[traindat,testdat,1.5,10]]

def preprocessor_prunevarsubmean_modular (fm_train_real=traindat,fm_test_real=testdat,width=1.4,size_cache=10):
	from modshogun import Chi2Kernel
	from modshogun import RealFeatures
	from modshogun import PruneVarSubMean

	feats_train=RealFeatures(fm_train_real)
	feats_test=RealFeatures(fm_test_real)

	preproc=PruneVarSubMean()
	preproc.init(feats_train)
	feats_train.add_preprocessor(preproc)
	feats_train.apply_preprocessor()
	feats_test.add_preprocessor(preproc)
	feats_test.apply_preprocessor()

	kernel=Chi2Kernel(feats_train, feats_train, width, size_cache)

	km_train=kernel.get_kernel_matrix()
	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()

	return km_train,km_test,kernel

if __name__=='__main__':
	print('PruneVarSubMean')
	preprocessor_prunevarsubmean_modular(*parameter_list[0])



"""
Utilities for testing
"""

from numpy import double, ushort, ubyte, matrix
from sg import sg

SIZE_CACHE=10


def check_accuracy (accuracy, **kwargs):
	acc=double(accuracy)
	output=[]

	for key, val in kwargs.iteritems():
		if val is not None:
			output.append('%s: %e' % (key, val))
	print ', '.join(output)+' <--- accuracy: %e' % accuracy

	for val in kwargs.itervalues():
		if val>acc:
			return False

	return True


def get_args (indata, prefix=''):
	# python dicts are not ordered, so we have to look at the number in
	# the parameter's name and insert items appropriately into an
	# ordered list

	# need to pregenerate list for using indices in loop
	ident=prefix+'arg'
	args=len(indata)*[None]

	for i in indata:
		if i.find(ident)==-1:
			continue

		try:
			idx=int(i[len(ident)])
		except ValueError:
			raise ValueError, 'Wrong indata data %s: "%s"!' % (ident, i)

		if i.find('_distance')!=-1: # DistanceKernel
			args[idx]=indata[i]
		else:
			try:
				args[idx]=eval(indata[i])
			except TypeError: # no bool
				args[idx]=indata[i]

	# weed out superfluous Nones
	return filter(lambda arg: arg is not None, args)


def set_features (indata, prefix):
	uppername=indata[prefix+'name'].upper()
	if uppername=='COMBINED' or uppername=='CUSTOM':
		return

	if uppername.startswith('SPARSE'):
		raise NotImplementedError, 'Sparse features not supported yet.'
	elif indata.has_key(prefix+'type') and \
		indata[prefix+'type']=='linear':
		raise NotImplementedError, 'Linear classifiers with sparse features not supported yet.'

	indata_train=indata[prefix+'data_train']
	indata_test=indata[prefix+'data_test']

	if indata.has_key(prefix+'alphabet'):
		alphabet=indata[prefix+'alphabet']
		if alphabet=='RAWBYTE':
			raise NotImplementedError, 'Alphabet RAWBYTE not supported yet.'

		if alphabet=='RAWDNA':
			data_train=list(indata_train[0])
			sg('set_features', 'TRAIN', data_train, 'DNA')
			sg('convert', 'TRAIN', 'STRING', 'CHAR', 'STRING', 'BYTE')
			data_test=list(indata_test[0])
			sg('set_features', 'TEST', data_test, 'DNA')
			sg('convert', 'TEST', 'STRING', 'CHAR', 'STRING', 'BYTE')
		elif alphabet=='CUBE':
			data_train=[str(x) for x in list(indata_train[0])]
			sg('set_features', 'TRAIN', data_train, alphabet)
			data_test=[str(x) for x in list(indata_test[0])]
			sg('set_features', 'TEST', data_test, alphabet)
		else:
			data_train=list(indata_train[0])
			sg('set_features', 'TRAIN', data_train, alphabet)
			data_test=list(indata_test[0])
			sg('set_features', 'TEST', data_test, alphabet)

	elif indata.has_key('data'): # CustomKernel
		sg('set_features', 'TRAIN',
			indata[prefix+'data'].astype(ushort))
		sg('set_features', 'TEST',
			indata[prefix+'data'].astype(ushort))

	else:
		as_types={
			'Word': ushort,
			'Real': double,
			'Byte': ubyte
		}
		as_type=as_types[indata[prefix+'feature_type']]
		sg('set_features', 'TRAIN', indata_train.astype(as_type))
		sg('set_features', 'TEST', indata_test.astype(as_type))

	convert_features_and_add_preproc(indata, prefix)


def set_and_train_distance (indata, do_train=True):
	prefix='distance_'
	dargs=get_args(indata, prefix)
	dname=fix_distance_name_inconsistency(indata[prefix+'name'])
	sg('set_distance', dname, indata[prefix+'feature_type'].upper(), *dargs)

def set_and_train_kernel (indata, do_train=True):
	prefix='kernel_'
	kargs=get_args(indata, prefix)
	kname=fix_kernel_name_inconsistency(indata[prefix+'name'])

	if indata.has_key(prefix+'arg0_size'):
		size=kargs[0]
		kargs=kargs[1:]
	else:
		size=SIZE_CACHE

	if kname=='POLY' and indata.has_key(prefix+'normalizer'):
		kargs.append(True)

	if kname=='DISTANCE':
		dname=fix_distance_name_inconsistency(kargs.pop())
		# FIXME: REAL is cheating and will break in the future
		sg('set_distance', dname, 'REAL')
		sg('set_kernel', kname, size, *kargs)
	else:
		sg('set_kernel', kname, indata[prefix+'feature_type'].upper(), size,
			*kargs)

def convert_features_and_add_preproc (indata, prefix):
	# having order implies having gap and reverse
	if not indata.has_key(prefix+'order'):
		return

	if indata[prefix+'feature_type']=='Ulong':
		type='ULONG'
	elif indata[prefix+'feature_type']=='Word':
		type='WORD'
	else:
		return

	order=indata[prefix+'order']
	sg('add_preproc', 'SORT'+type+'STRING')
	sg('convert', 'TRAIN', 'STRING', 'CHAR', 'STRING', type,
		order, order-1, indata[prefix+'gap'], indata[prefix+'reverse'])
	sg('attach_preproc', 'TRAIN')

	sg('convert', 'TEST', 'STRING', 'CHAR', 'STRING', type,
		order, order-1, indata[prefix+'gap'], indata[prefix+'reverse'])
	sg('attach_preproc', 'TEST')


# fix inconsistency in modular/static interfaces
def fix_kernel_name_inconsistency (kname):
	kname=kname.upper()
	if kname=='LOCALITYIMPROVEDSTRING':
		return 'LIK'
	elif kname=='SIMPLELOCALITYIMPROVEDSTRING':
		return 'SLIK'
	elif kname=='WORDMATCH':
		return 'MATCH'
	elif kname=='WEIGHTEDDEGREEPOSITIONSTRING':
		return 'WEIGHTEDDEGREEPOS'
	elif kname=='COMMULONGSTRING':
		return 'COMMSTRING'
	elif kname=='COMMWORDSTRING':
		return 'COMMSTRING'
	elif kname=='WEIGHTEDCOMMWORDSTRING':
		return 'WEIGHTEDCOMMSTRING'
	elif kname.endswith('WORDSTRING'):
		return kname.split('WORDSTRING')[0]
	elif kname.endswith('STRING'):
		return kname.split('STRING')[0]
	elif kname.endswith('WORD'):
		return kname.split('WORD')[0]
	else:
		return kname


def fix_distance_name_inconsistency (dname):
	dname=dname.upper()
	if dname.endswith('WORDDISTANCE'):
		return dname.split('WORDDISTANCE')[0]
	elif dname.endswith('DISTANCE'):
		return dname.split('DISTANCE')[0]
	elif dname.endswith('METRIC'):
		return dname.split('METRIC')[0]
	else:
		return dname


def fix_classifier_name_inconsistency (cname):
	cname=cname.upper()
	if cname.startswith('LIBSVM') and len(cname)>len('LIBSVM'):
		return 'LIBSVM_'+cname.split('LIBSVM')[1]
	else:
		return cname


def fix_clustering_name_inconsistency (cname):
	return cname.upper()

def fix_preproc_name_inconsistency (pname):
	return pname.upper()

def fix_regression_name_inconsistency (rname):
	return rname.upper()



#!/usr/bin/env python
traindat = '../data/fm_train_real.dat'
testdat = '../data/fm_test_real.dat'

parameter_list = [[traindat,testdat],[traindat,testdat]]

def distance_sparseeuclidean_modular (train_fname=traindat,test_fname=testdat):
	from modshogun import RealFeatures, SparseRealFeatures, SparseEuclideanDistance, CSVFile

	realfeat=RealFeatures(CSVFile(train_fname))
	feats_train=SparseRealFeatures()
	feats_train.obtain_from_simple(realfeat)
	realfeat=RealFeatures(CSVFile(test_fname))
	feats_test=SparseRealFeatures()
	feats_test.obtain_from_simple(realfeat)

	distance=SparseEuclideanDistance(feats_train, feats_train)

	dm_train=distance.get_distance_matrix()
	distance.init(feats_train, feats_test)
	dm_test=distance.get_distance_matrix()

	return distance,dm_train,dm_test

if __name__=='__main__':
	print('SparseEuclideanDistance')
	distance_sparseeuclidean_modular(*parameter_list[0])



#!/usr/bin/env python

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Written (W) 2013 Viktor Gal
# Copyright (C) 2013 Viktor Gal

def get_class_list_content(class_list_file):
    f = open(class_list_file, 'r')
    content = f.readlines()
    f.close()
    return content

def get_class_list(class_list_content):
    import re
    in_class_list = False
    classes = []
    template_classes = []
    for line in class_list_content:
            if line == '\n':
                continue
            l = [l.strip() for l in line.split()]
            if 'class_list[]' in l:
                in_class_list = True
                continue

            if in_class_list:
                if '};' in l:
                    in_class_list = False
                    continue
                result = re.match(r"{\"(?P<sho_class>\w+)\"", l[0])

                if result:
                    sho_class=result.group('sho_class')
                    #print l[1]
                    if l[1] == 'SHOGUN_BASIC_CLASS':
                        classes.append(sho_class)
                    if l[1] == 'SHOGUN_TEMPLATE_CLASS':
                        template_classes.append(sho_class)
    return classes, template_classes

def entry(templateFile, class_list_file):
    templateLoader = jinja2.FileSystemLoader(searchpath="/")
    templateEnv = jinja2.Environment(loader=templateLoader)

    template = templateEnv.get_template(templateFile)

    # get the content of class_list.cpp
    class_list_content = get_class_list_content(class_list_file)

    classes, template_classes = get_class_list(class_list_content)

    types = ['PT_BOOL', 'PT_CHAR', 'PT_INT8', 'PT_UINT8', 'PT_INT16', 'PT_UINT16', 'PT_INT32',
             'PT_UINT32', 'PT_INT64', 'PT_UINT64', 'PT_FLOAT32', 'PT_FLOAT64', 'PT_FLOATMAX']

    templateVars = {"classes" : classes, "template_classes" : template_classes, "types" : types}

    return template.render(templateVars)

# execution
# ./clone_unittest.cc.py <template file> <output file name> <extra args...>

import sys
TEMPLATE_FILE = sys.argv[1]
output_file = sys.argv[2]
class_list_file = sys.argv[3]

try:
    import jinja2
    outputText = entry(TEMPLATE_FILE, class_list_file)
except ImportError:
    import os
    basename = os.path.basename(output_file)
    basename = basename.replace('.cc', '')
    print("Please install jinja2 for clone unit-tests");
    outputText = ['''#include <gtest/gtest.h>
TEST(Dummy, %s_dummy)
{
}''' % (basename)]

f = open(output_file, 'w')
f.writelines(outputText)
f.close()



#!/usr/bin/env python

traindat = '../data/fm_train_real.dat'
testdat = '../data/fm_test_real.dat'
label_traindat = '../data/label_train_twoclass.dat'

parameter_list= [[traindat,testdat,label_traindat],[traindat,testdat,label_traindat]]


def kernel_combined_custom_poly_modular (train_fname = traindat,test_fname = testdat,train_label_fname=label_traindat):
    from modshogun import CombinedFeatures, RealFeatures, BinaryLabels
    from modshogun import CombinedKernel, PolyKernel, CustomKernel
    from modshogun import LibSVM, CSVFile

    kernel = CombinedKernel()
    feats_train = CombinedFeatures()

    tfeats = RealFeatures(CSVFile(train_fname))
    tkernel = PolyKernel(10,3)
    tkernel.init(tfeats, tfeats)
    K = tkernel.get_kernel_matrix()
    kernel.append_kernel(CustomKernel(K))

    subkfeats_train = RealFeatures(CSVFile(train_fname))
    feats_train.append_feature_obj(subkfeats_train)
    subkernel = PolyKernel(10,2)
    kernel.append_kernel(subkernel)

    kernel.init(feats_train, feats_train)

    labels = BinaryLabels(CSVFile(train_label_fname))
    svm = LibSVM(1.0, kernel, labels)
    svm.train()

    kernel = CombinedKernel()
    feats_pred = CombinedFeatures()

    pfeats = RealFeatures(CSVFile(test_fname))
    tkernel = PolyKernel(10,3)
    tkernel.init(tfeats, pfeats)
    K = tkernel.get_kernel_matrix()
    kernel.append_kernel(CustomKernel(K))

    subkfeats_test = RealFeatures(CSVFile(test_fname))
    feats_pred.append_feature_obj(subkfeats_test)
    subkernel = PolyKernel(10, 2)
    kernel.append_kernel(subkernel)
    kernel.init(feats_train, feats_pred)

    svm.set_kernel(kernel)
    svm.apply()
    km_train=kernel.get_kernel_matrix()
    return km_train,kernel

if __name__=='__main__':
    kernel_combined_custom_poly_modular(*parameter_list[0])



#!/usr/bin/env python
from tools.load import LoadMatrix
lm=LoadMatrix()

traindat = lm.load_dna('../data/fm_train_dna.dat')
testdat = lm.load_dna('../data/fm_test_dna.dat')
parameter_list = [[traindat,testdat,4,0,False, False],[traindat,testdat,4,0,False,False]]

def kernel_comm_word_string_modular (fm_train_dna=traindat, fm_test_dna=testdat, order=3, gap=0, reverse = False, use_sign = False):

	from modshogun import CommWordStringKernel
	from modshogun import StringWordFeatures, StringCharFeatures, DNA
	from modshogun import SortWordString

	charfeat=StringCharFeatures(DNA)
	charfeat.set_features(fm_train_dna)
	feats_train=StringWordFeatures(charfeat.get_alphabet())
	feats_train.obtain_from_char(charfeat, order-1, order, gap, reverse)
	preproc=SortWordString()
	preproc.init(feats_train)
	feats_train.add_preprocessor(preproc)
	feats_train.apply_preprocessor()

	charfeat=StringCharFeatures(DNA)
	charfeat.set_features(fm_test_dna)
	feats_test=StringWordFeatures(charfeat.get_alphabet())
	feats_test.obtain_from_char(charfeat, order-1, order, gap, reverse)
	feats_test.add_preprocessor(preproc)
	feats_test.apply_preprocessor()

	kernel=CommWordStringKernel(feats_train, feats_train, use_sign)

	km_train=kernel.get_kernel_matrix()
	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return km_train,km_test,kernel

if __name__=='__main__':
	print('CommWordString')
	kernel_comm_word_string_modular(*parameter_list[0])



#!/usr/bin/env python
from tools.multiclass_shared import prepare_data

[traindat, label_traindat, testdat, label_testdat] = prepare_data(False)

parameter_list = [[traindat,testdat,label_traindat,label_testdat,2.1,1,1e-5],[traindat,testdat,label_traindat,label_testdat,2.2,1,1e-5]]

def classifier_multiclassmultipleoutputliblinear_modular (fm_train_real=traindat,fm_test_real=testdat,label_train_multiclass=label_traindat,label_test_multiclass=label_testdat,width=2.1,C=1,epsilon=1e-5):
	from modshogun import RealFeatures, MulticlassLabels, MulticlassMultipleOutputLabels
	from modshogun import MulticlassLibLinear

	feats_train=RealFeatures(fm_train_real)
	feats_test=RealFeatures(fm_test_real)

	labels=MulticlassLabels(label_train_multiclass)

	classifier = MulticlassLibLinear(C,feats_train,labels)
	classifier.train()

	label_pred = classifier.apply_multiclass_multiple_output(feats_test,2)
	out = label_pred.get_labels()
	#print out
	return out

if __name__=='__main__':
	print('MulticlassMultipleOutputLibLinear')
	classifier_multiclassmultipleoutputliblinear_modular(*parameter_list[0])



#!/usr/bin/env python
from tools.load import LoadMatrix
lm=LoadMatrix()
traindat = lm.load_dna('../data/fm_train_dna.dat')
testdat = lm.load_dna('../data/fm_test_dna.dat')

parameter_list = [[traindat,testdat,5,5],[traindat,testdat,6,6]]

def kernel_distantsegments_modular (fm_train_dna=traindat,fm_test_dna=testdat,delta=5, theta=5):
	from modshogun import StringCharFeatures, DNA
	from modshogun import DistantSegmentsKernel

	feats_train=StringCharFeatures(fm_train_dna, DNA)
	feats_test=StringCharFeatures(fm_test_dna, DNA)

	kernel=DistantSegmentsKernel(feats_train, feats_train, 10, delta, theta)

	km_train=kernel.get_kernel_matrix()
	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()

	return km_train, km_test, kernel


if __name__=='__main__':
	print('DistantSegments')
	kernel_distantsegments_modular(*parameter_list[0])



from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()

traindat=lm.load_numbers('../data/fm_train_real.dat')
testdat=lm.load_numbers('../data/fm_test_real.dat')
parameter_list=[[traindat,testdat,1.,10],[traindat,testdat,1.5,11]]

def kernel_combined(fm_train_real=traindat,fm_test_real=testdat,
		 weight=1.,size_cache=10):
	sg('clean_kernel')
	sg('clean_features', 'TRAIN')
	sg('clean_features', 'TEST')
	sg('set_kernel', 'COMBINED', size_cache)
	sg('add_kernel', weight, 'LINEAR', 'REAL', size_cache)
	sg('add_features', 'TRAIN', fm_train_real)
	sg('add_features', 'TEST', fm_test_real)
	sg('add_kernel', weight, 'GAUSSIAN', 'REAL', size_cache, 1.)
	sg('add_features', 'TRAIN', fm_train_real)
	sg('add_features', 'TEST', fm_test_real)
	sg('add_kernel', weight, 'POLY', 'REAL', size_cache, 3, False)
	sg('add_features', 'TRAIN', fm_train_real)
	sg('add_features', 'TEST', fm_test_real)

	km=sg('get_kernel_matrix', 'TRAIN')
	km=sg('get_kernel_matrix', 'TEST')
	return km

if __name__=='__main__':
	print('Combined')
	kernel_combined(*parameter_list[0])



import numpy
numpy.random.seed(40)
tt = numpy.genfromtxt('../../data/toy/swissroll_color.dat',unpack=True).T
X = numpy.genfromtxt('../../data/toy/swissroll.dat',unpack=True).T
N = X.shape[1]
converters = []

from shogun.Converter import LocallyLinearEmbedding
lle = LocallyLinearEmbedding()
lle.set_k(9)
converters.append((lle, "LLE with k=%d" % lle.get_k()))

from shogun.Converter import MultidimensionalScaling
mds = MultidimensionalScaling()
converters.append((mds, "Classic MDS"))

lmds = MultidimensionalScaling()
lmds.set_landmark(True)
lmds.set_landmark_number(20)
converters.append((lmds,"Landmark MDS with %d landmarks" % lmds.get_landmark_number()))

from shogun.Converter import Isomap
cisomap = Isomap()
cisomap.set_k(9)
converters.append((cisomap,"Isomap with k=%d" % cisomap.get_k()))

from shogun.Converter import DiffusionMaps
from shogun.Kernel import GaussianKernel
dm = DiffusionMaps()
dm.set_t(2)
dm.set_width(1000.0)
converters.append((dm,"Diffusion Maps with t=%d, sigma=%.1f" % (dm.get_t(),dm.get_width())))

from shogun.Converter import HessianLocallyLinearEmbedding
hlle = HessianLocallyLinearEmbedding()
hlle.set_k(6)
converters.append((hlle,"Hessian LLE with k=%d" % (hlle.get_k())))

from shogun.Converter import LocalTangentSpaceAlignment
ltsa = LocalTangentSpaceAlignment()
ltsa.set_k(6)
converters.append((ltsa,"LTSA with k=%d" % (ltsa.get_k())))

from shogun.Converter import LaplacianEigenmaps
le = LaplacianEigenmaps()
le.set_k(20)
le.set_tau(100.0)
converters.append((le,"Laplacian Eigenmaps with k=%d, tau=%d" % (le.get_k(),le.get_tau())))

import matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D

fig = plt.figure()

new_mpl = False

try:
	swiss_roll_fig = fig.add_subplot(3,3,1, projection='3d')
	new_mpl = True
except:
	figure = plt.figure()
	swiss_roll_fig = Axes3D(figure)

swiss_roll_fig.scatter(X[0], X[1], X[2], s=10, c=tt, cmap=plt.cm.Spectral)
swiss_roll_fig._axis3don = False
plt.suptitle('Swissroll embedding',fontsize=9)
plt.subplots_adjust(hspace=0.4)

from shogun.Features import RealFeatures

for (i, (converter, label)) in enumerate(converters):
	X = numpy.genfromtxt('../../data/toy/swissroll.dat',unpack=True).T
	features = RealFeatures(X)
	converter.set_target_dim(2)
	converter.parallel.set_num_threads(1)
	new_feats = converter.embed(features).get_feature_matrix()
	if not new_mpl:
		embedding_subplot = fig.add_subplot(4,2,i+1)
	else:
		embedding_subplot = fig.add_subplot(3,3,i+2)
	embedding_subplot.scatter(new_feats[0],new_feats[1], c=tt, cmap=plt.cm.Spectral)
	plt.axis('tight')
	plt.xticks([]), plt.yticks([])
	plt.title(label,fontsize=9)
	print converter.get_name(), 'done'

plt.show()



#!/usr/bin/env python
from tools.load import LoadMatrix
lm=LoadMatrix()
traindat = lm.load_dna('../data/fm_train_dna.dat')
testdat = lm.load_dna('../data/fm_test_dna.dat')

parameter_list = [[traindat,testdat,3,False],[traindat,testdat,4,False]]
def kernel_poly_match_string_modular (fm_train_dna=traindat,fm_test_dna=testdat,degree=3,inhomogene=False):
	from modshogun import PolyMatchStringKernel
	from modshogun import StringCharFeatures, DNA

	feats_train=StringCharFeatures(fm_train_dna, DNA)
	feats_test=StringCharFeatures(fm_train_dna, DNA)

	kernel=PolyMatchStringKernel(feats_train, feats_train, degree, inhomogene)

	km_train=kernel.get_kernel_matrix()
	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return km_train,km_test,kernel

if __name__=='__main__':
	print('PolyMatchString')
	kernel_poly_match_string_modular(*parameter_list[0])



"""
Test Kernel
"""

from sg import sg
from numpy import double
import util


def _evaluate_kernel (indata, prefix, kernel_is_set=False):
	if not kernel_is_set:
		util.set_and_train_kernel(indata)

	kmatrix=sg('get_kernel_matrix', 'TRAIN')
	km_train=max(abs(indata[prefix+'matrix_train']-kmatrix).flat)

	kmatrix=sg('get_kernel_matrix', 'TEST')
	km_test=max(abs(indata[prefix+'matrix_test']-kmatrix).flat)

	return util.check_accuracy(
		indata[prefix+'accuracy'], km_train=km_train, km_test=km_test)


def _get_subkernels (indata, prefix):
	subkernels={}
	prefix=prefix+'subkernel'
	len_prefix=len(prefix)

	# loop through indata (unordered dict) to gather subkernel data
	for key in indata:
		if key.find(prefix)==-1:
			continue

		# get subkernel's number
		try:
			num=key[len_prefix]
		except ValueError:
			raise ValueError, 'Cannot find number for subkernel: "%s"!' % data

		# get item's name
		name=key[len_prefix+2:]

		# append new item
		if not subkernels.has_key(num):
			subkernels[num]={}
		subkernels[num][name]=indata[key]

	# got all necessary information in new structure, now create a kernel
	# object for each subkernel
	for num, data in subkernels.iteritems():
		subkernels[num]['args']=util.get_args(data, '')
		subkernels[num]['name']=util.fix_kernel_name_inconsistency(data['name'])

	return subkernels


def _evaluate_combined (indata, prefix):
	sg('set_kernel', 'COMBINED', util.SIZE_CACHE)

	subkernels=_get_subkernels(indata, prefix)
	for subk in subkernels.itervalues():
		sg('add_kernel', 1., subk['name'], subk['feature_type'].upper(),
			*subk['args'])

		if subk.has_key('alphabet'):
			sg('add_features', 'TRAIN',
				list(subk['data_train'][0]), subk['alphabet'])
			sg('add_features', 'TEST',
				list(subk['data_test'][0]), subk['alphabet'])
		else:
			sg('add_features', 'TRAIN', subk['data_train'][0])
			sg('add_features', 'TEST', subk['data_test'][0])

	return _evaluate_kernel(indata, prefix, True)


def _evaluate_auc (indata, prefix):
	raise NotImplementedError, 'AUC kernel not yet supported in static interfaces.'

	subk=_get_subkernels(indata, prefix)[0]
	subk['kernel'].init(feats_subk['train'], feats_subk['test'])

	feats={
		'train': WordFeatures(indata['data_train'].astype(ushort)),
		'test': WordFeatures(indata['data_test'].astype(ushort))}
	kernel=AUCKernel(10, subk['kernel'])

	return _evaluate_kernel(indata, prefix, True)


def _evaluate_custom (indata, prefix):
	raise NotImplementedError, 'Custom kernel not yet implemented in static interfaces.'

	symdata=indata[prefix+'symdata']
	lowertriangle=array([symdata[(x,y)] for x in xrange(symdata.shape[1])
		for y in xrange(symdata.shape[0]) if y<=x])

	sg('set_kernel', 'CUSTOM')
	sg('set_triangle_kernel_matrix_from_triangle', lowertriangle)
	triangletriangle=max(abs(
		indata[prefix+'matrix_triangletriangle']-sg('get_kernel_matrix')).flat)

	sg('set_triangle_kernel_matrix_from_full', indata[prefix+'symdata'])
	fulltriangle=max(abs(
		indata[prefix+'matrix_fulltriangle']-sg('get_kernel_matrix')).flat)

	sg('set_full_kernel_matrix_from_full', indata[prefix+'data'])
	fullfull=max(abs(
		indata[prefix+'matrix_fullfull']-sg('get_kernel_matrix')).flat)

	return util.check_accuracy(
		indata[prefix+'accuracy'],
		triangletriangle=triangletriangle,
		fulltriangle=fulltriangle,
		fullfull=fullfull
	)


def _evaluate_pie (indata, prefix):
	pseudo_pos=1e-10
	pseudo_neg=1e-10

	sg('new_plugin_estimator', pseudo_pos, pseudo_neg)

	sg('set_labels', 'TRAIN', double(indata['classifier_labels']))
	sg('train_estimator')

	return _evaluate_kernel(indata, prefix)


def _evaluate_top_fisher (indata, prefix):
	raise NotImplementedError, 'TOP/Fisher not yet supported in static interfaces.'

	sg('new_hmm', indata[prefix+'N'], indata[prefix+'M'])
	pos=HMM(wordfeats['train'], indata[prefix+'N'], indata[prefix+'M'],
		indata[prefix+'pseudo'])
	pos.train()
	pos.baum_welch_viterbi_train(BW_NORMAL)
	neg=HMM(wordfeats['train'], indata[prefix+'N'], indata[prefix+'M'],
		indata[prefix+'pseudo'])
	neg.train()
	neg.baum_welch_viterbi_train(BW_NORMAL)
	pos_clone=HMM(pos)
	neg_clone=HMM(neg)
	pos_clone.set_observations(wordfeats['test'])
	neg_clone.set_observations(wordfeats['test'])

	if indata[prefix+'type']=='TOP':
		feats['train']=TOPFeatures(10, pos, neg, False, False)
		feats['test']=TOPFeatures(10, pos_clone, neg_clone, False, False)
	else:
		feats['train']=FKFeatures(10, pos, neg)
		feats['train'].set_opt_a(-1) #estimate prior
		feats['test']=FKFeatures(10, pos_clone, neg_clone)
		feats['test'].set_a(feats['train'].get_a()) #use prior from training data

	prefix='kernel_'
	args=util.get_args(indata, prefix)
	kernel=PolyKernel(feats['train'], feats['train'], *args)
	km_train=max(abs(
		indata[prefix+'matrix_train']-kernel.get_kernel_matrix()).flat)
	kernel.init(feats['train'], feats['test'])
	km_test=max(abs(
		indata[prefix+'matrix_test']-kernel.get_kernel_matrix()).flat)

	return util.check_accuracy(indata[prefix+'accuracy'],
		km_train=km_train, km_test=km_test)


########################################################################
# public
########################################################################

def test (indata):
	prefix='topfk_'
	if indata.has_key(prefix+'name'):
		try:
			util.set_features(indata, prefix)
			return _evaluate_top_fisher(indata, prefix)
		except NotImplementedError, e:
			print e
			return True

	prefix='kernel_'
	try:
		util.set_features(indata, prefix)
		names=['Combined', 'AUC', 'Custom']
		for name in names:
			if indata[prefix+'name']==name:
				return eval('_evaluate_'+name.lower()+'(indata, prefix)')

		names=['HistogramWordString', 'SalzbergWordString']
		for name in names:
			if indata[prefix+'name']==name:
				return _evaluate_pie(indata, prefix)

		# pretty normal kernel
		return _evaluate_kernel(indata, prefix)
	except NotImplementedError, e:
		print e
		return True





#!/usr/bin/env python
from modshogun import CombinedFeatures, RealFeatures, BinaryLabels
from modshogun import CombinedKernel, PolyKernel, CustomKernel
from modshogun import MKLClassification
from tools.load import LoadMatrix
lm=LoadMatrix()

traindat = lm.load_numbers('../data/fm_train_real.dat')
testdat = lm.load_numbers('../data/fm_test_real.dat')
label_traindat = lm.load_labels('../data/label_train_twoclass.dat')

parameter_list = [[traindat,testdat,label_traindat],[traindat,testdat,label_traindat]]
#    fm_train_real.shape
#    fm_test_real.shape
#    combined_custom()

def mkl_binclass_modular (fm_train_real=traindat,fm_test_real=testdat,fm_label_twoclass = label_traindat):

    ##################################
    # set up and train

    # create some poly train/test matrix
    tfeats = RealFeatures(fm_train_real)
    tkernel = PolyKernel(10,3)
    tkernel.init(tfeats, tfeats)
    K_train = tkernel.get_kernel_matrix()

    pfeats = RealFeatures(fm_test_real)
    tkernel.init(tfeats, pfeats)
    K_test = tkernel.get_kernel_matrix()

    # create combined train features
    feats_train = CombinedFeatures()
    feats_train.append_feature_obj(RealFeatures(fm_train_real))

    # and corresponding combined kernel
    kernel = CombinedKernel()
    kernel.append_kernel(CustomKernel(K_train))
    kernel.append_kernel(PolyKernel(10,2))
    kernel.init(feats_train, feats_train)

    # train mkl
    labels = BinaryLabels(fm_label_twoclass)
    mkl = MKLClassification()

    # which norm to use for MKL
    mkl.set_mkl_norm(1) #2,3

    # set cost (neg, pos)
    mkl.set_C(1, 1)

    # set kernel and labels
    mkl.set_kernel(kernel)
    mkl.set_labels(labels)

    # train
    mkl.train()
    #w=kernel.get_subkernel_weights()
    #kernel.set_subkernel_weights(w)


    ##################################
    # test

    # create combined test features
    feats_pred = CombinedFeatures()
    feats_pred.append_feature_obj(RealFeatures(fm_test_real))

    # and corresponding combined kernel
    kernel = CombinedKernel()
    kernel.append_kernel(CustomKernel(K_test))
    kernel.append_kernel(PolyKernel(10, 2))
    kernel.init(feats_train, feats_pred)

    # and classify
    mkl.set_kernel(kernel)
    mkl.apply()
    return mkl.apply(),kernel

if __name__=='__main__':
    mkl_binclass_modular (*parameter_list[0])



#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Written (W) 2006-2009 Soeren Sonnenburg
# Written (W) 2006-2007 Mikio Braun
# Copyright (C) 2007 Fraunhofer Institute FIRST and Max-Planck-Society

import time
from string import maketrans

class ordered_dict(dict):
    """
    Provide an ordered dictionary with chromosome identifiers.
    """
    def __init__(self, *args, **kwargs):
        dict.__init__(self, *args, **kwargs)
        self._order = self.keys()

    def __setitem__(self, key, value):
        dict.__setitem__(self, key, value)
        if key in self._order:
            self._order.remove(key)
        self._order.append(key)

    def __delitem__(self, key):
        dict.__delitem__(self, key)
        self._order.remove(key)

    def ordered_items(self):
        return [(key,self[key]) for key in self._order]


""" read a table browser ascii output file (http://genome.ucsc.edu/cgi-bin/hgTables) """
def read_table_browser(f):
	table=dict();
	for l in f.readlines():
		if not l.startswith('#'):
			(name,chrom,strand,txStart,txEnd,cdsStart,cdsEnd,exonCount,exonStarts,exonEnds,proteinID,alignID)=l.split('\t')
			exonStarts=[ int(i) for i in exonStarts.split(',')[:-1] ]
			exonEnds=[ int(i) for i in exonEnds.split(',')[:-1] ]

			table[name]={ 'chrom': chrom, 'strand': strand, 'txStart': int(txStart), 'txEnd': int(txEnd),
			'cdsStart': int(cdsStart), 'cdsEnd': int(cdsEnd), 'exonCount': int(exonCount), 'exonStarts': exonStarts,
			'exonEnds': exonEnds, 'proteinID': proteinID, 'alignID': alignID[:-1] }

	return table

""" get promoter region """
def get_promoter_region(chromosome, strand, gene_start, gene_end, genome, length):

	if strand == '+':
		return load_genomic(chromosome, strand, gene_start, gene_start+length, genome, one_based=False)
	elif strand == '-':
		return load_genomic(chromosome, strand, gene_end, gene_end+length, genome, one_based=False)
	else:
		print 'unknown strand'
		return None

""" reverse + complement a DNA sequence (only letters ACGT are translated!)
	FIXME won't work with all the rest like y... """
def reverse_complement(str):
	t=maketrans('acgtACGT','tgcaTGCA')
	return str[len(str)::-1].translate(t)

""" works only with .fa files that contain a single entry """
def read_single_fasta(fname):
	str=file(fname).read()
	str=str[str.index('\n')+1:].replace('\n','')
	return str

""" writes only single enty .fa files """
def write_single_fasta(fname, name, str, linelen=60):
	header= '>' + name + '\n'
	f=file(fname,'a')
	f.write(header)
	for i in xrange(0,len(str),linelen):
		f.write(str[i:i+linelen]+'\n')
	f.close()

""" read fasta as dictionary """
def read_fasta(f):
	fasta=ordered_dict()
	fa=""
	key=None
	for s in f.readlines():
		if s.startswith('>'):
			if fa and key:
				fasta[key]=fa
			key=s[1:-1]
			fasta[key]=""
			fa=""
		else:
			fa+=s[:-1]

	if fa and key:
		fasta[key]=fa

	return fasta

def write_fasta(f, d, linelen=60):
    """ write dictionary fasta """
    for k in sorted(d):
        f.write('>%s\n' % k);
        s = d[k]
        for i in xrange(0, len(s), linelen):
            f.write(s[i:i+linelen] + '\n')

def write_gff_header(f, (source, version), (seqtype, seqname)):
	""" writes a gff version 2 file
		descrlist is a list of dictionaries, each of which contain these fields:
		<seqname> <source> <feature> <start> <end> <score> <strand> <frame> [attributes] [comments]
	"""
	f.write('##gff-version 2\n')
	f.write('##source-version %s %s\n' % (source, version) )

	t=time.localtime()
	f.write("##date %d-%d-%d %d:%d:%d\n" % t[0:6])

	f.write('##Type %s %s\n' % (seqtype, seqname) )

def write_gff_line(f, descr):
	d=descr
	f.write('%s\t%s\t%s\t%d\t%d\t%f\t%s\t%d' % (d['seqname'], d['source'],
										d['feature'], d['start'], d['end'],
										d['score'], d['strand'], d['frame']))
	if d.has_key('attributes'):
		f.write('\t' + d['attributes'])
		if d.has_key('comments'):
			f.write('\t' + d['comments'])
	f.write('\n')

def write_spf_header(f, (source, version), (seqtype, seqname)):
	""" writes a gff version 2 file
		descrlist is a list of dictionaries, each of which contain these fields:
		<seqname> <source> <feature> <start> <end> <score> <strand> <frame> [attributes] [comments]
	"""

	f.write('##spf-version 1\n')
	f.write('##source-version %s %s\n' % (source, version) )

	t=time.localtime()
	f.write("##date %d-%d-%d %d:%d:%d\n" % t[0:6])

	f.write('##Type %s %s\n' % (seqtype, seqname) )

def write_spf_line(f, descr):
	d=descr
	f.write('%s\t%s\t%s\t%d\t%s\t%f' % (d['seqname'], d['source'],
										d['feature'], d['position'],
										d['strand'], d['score']))
	if d.has_key('attributes'):
		f.write('\t' + d['attributes'])
		if d.has_key('comments'):
			f.write('\t' + d['comments'])
	f.write('\n')

def write_gff(f, (source, version), (seqtype, seqname), descrlist, skipheader=False):
	""" writes a gff version 2 file
		descrlist is a list of dictionaries, each of which contain these fields:
		<seqname> <source> <feature> <start> <end> <score> <strand> <frame> [attributes] [comments]
	"""

	if not skipheader:
		f.write('##gff-version 2\n')
		f.write('##source-version %s %s\n' % (source, version) )

		t=time.localtime()
		f.write("##date %d-%d-%d %d:%d:%d\n" % t[0:6])

	f.write('##Type %s %s\n' % (seqtype, seqname) )

	for d in descrlist:
		f.write('%s\t%s\t%s\t%d\t%d\t%f\t%s\t%d' % (d['seqname'], d['source'],
											d['feature'], d['start'], d['end'],
											d['score'], d['strand'], d['frame']))
		if d.has_key('attributes'):
			f.write('\t' + d['attributes'])
			if d.has_key('comments'):
				f.write('\t' + d['comments'])
		f.write('\n')




from numpy import *
import numpy as np
import sys
import math as m

class circle_data:
	def __init__(self):
		pass

	def  generate_data(self,number_of_points_for_circle1,number_of_points_for_circle2,row_vector):
		"""
		generate_circle_data generates a dataset in the shape of 2 circles.In this particular example we have taken the radius of
		the 2 circles to be 10 and 30.
		'number_of_points_for_circle1' and 'number_of_points_for_circle2' define the number of y
		coordinates for each circle.
		'row_vector': decides the dimension of the input data.In this particular example it is taken as 2
		'circle1':the first circle
		'circle2':the second circle
		'xmin_circle1':the minimum value of the x coordinate  for first circle
		'xmax_circle1':the maximum value of the x coordinate  for first circle
		'xmin_circle2':the minimum value of the x coordinate  for second circle
		'xmax_circle2':the maximum value of the x coordinate  for second circle
		'xrange_circle1': the horizontal range of the x ccordinates of first circle
		'xrange_circle2': the horizontal range of the x ccordinates of second circle
		'radius1':the radius of the first circle
		'radius2':the radius of the second circle
		'two_circles':the concatenated data of the 2 circles
		"""
		#number_of_points_for_circle1=42
		#number_of_points_for_circle1=122
		row_vector=2
		circle1=zeros((row_vector,number_of_points_for_circle1))
		circle2=zeros((row_vector,number_of_points_for_circle2))
		radius1=(number_of_points_for_circle1-2)/4
		radius2=(number_of_points_for_circle2-2)/4
		l2=len(circle2[0])
		l1=len(circle1[0])
		xmin_circle1=-1*radius1
		xmax_circle1=radius1
		xmin_circle2=-1*radius2
		xmax_circle2=radius2
		xrange_circle1=range(xmin_circle1,xmax_circle1+1)
		xrange_circle1=xrange_circle1+xrange_circle1
		xrange_circle2=range(xmin_circle2,xmax_circle2+1)
		xrange_circle2=xrange_circle2+xrange_circle2
		circle1[0][:]=xrange_circle1
		circle2[0][:]=xrange_circle2
		mat1=ones((1,number_of_points_for_circle1))
		mat2=ones((1,number_of_points_for_circle2))
		mat1=radius1*radius1*ones((1,number_of_points_for_circle1))
		mat2=radius2*radius2*ones((1,number_of_points_for_circle2))
		circle1[1][:]=mat1-(circle1[0][:]*circle1[0][:])
		circle2[1][:]=mat2-(circle2[0][:]*circle2[0][:])

		circle1[1][:]=[m.sqrt(circle1[1][i]) for i in range(0,number_of_points_for_circle1)]
		circle1[1][(number_of_points_for_circle1/2):]=-1*circle1[1][(number_of_points_for_circle1/2):]

		circle2[1][:]=[m.sqrt(circle2[1][i]) for i in range(0,number_of_points_for_circle2)]
		circle2[1][(number_of_points_for_circle2/2):]=-1*circle2[1][(number_of_points_for_circle2/2):]

		two_circles=hstack((circle1,circle2))

		return two_circles



"""
Test Clustering
"""

from modshogun import EuclideanDistance
from modshogun import *

import util


def _evaluate (indata):
	if 'clustering_k' in indata:
		first_arg=indata['clustering_k']
	elif 'clustering_merges' in indata:
		first_arg=indata['clustering_merges']
	else:
		return False

	feats=util.get_features(indata, 'distance_')
	dfun=eval(indata['distance_name'])
	distance=dfun(feats['train'], feats['train'])

	cfun=eval(indata['clustering_name'])
	clustering=cfun(first_arg, distance)
	clustering.train()

	if 'clustering_radi' in indata:
		radi=max(abs(clustering.get_radiuses()-indata['clustering_radi']))
		centers=max(abs(clustering.get_cluster_centers().flatten() - \
			indata['clustering_centers'].flat))
		return util.check_accuracy(indata['clustering_accuracy'],
			radi=radi, centers=centers)
	elif 'clustering_merge_distance' in indata:
		merge_distance=max(abs(clustering.get_merge_distances()- \
			indata['clustering_merge_distance']))
		pairs=max(abs(clustering.get_cluster_pairs()- \
			indata['clustering_pairs']).flat)
		return util.check_accuracy(indata['clustering_accuracy'],
			merge_distance=merge_distance, pairs=pairs)
	else:
		return util.check_accuracy(indata['clustering_accuracy'])


########################################################################
# public
########################################################################

def test (indata):
	return _evaluate(indata)




import modshogun as sg
import data

# load data
feature_matrix = data.swissroll()
# create features instance
features = sg.RealFeatures(feature_matrix)

# create Locally Linear Embedding converter instance
converter = sg.LocallyLinearEmbedding()

# set target dimensionality
converter.set_target_dim(2)
# set number of neighbors
converter.set_k(10)
# set reconstruction shift (optional)
converter.set_reconstruction_shift(1e-3)
# set nullspace shift (optional)
converter.set_nullspace_shift(-1e-6)

# compute embedding with Locally Linear Embedding method
embedding_first = converter.embed(features)

# set number of neighbors to be used
converter.set_k(50)

# compute embedding with Locally Linear Embedding method
embedding_second = converter.embed(features)



#!/usr/bin/env python

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Written (W) 2011 Sergey Lisitsyn
# Copyright (C) 2011 Sergey Lisitsyn

from numpy import *
from pylab import *
from modshogun import *
import random
import difflib

def word_kernel(words):
	N = len(words)
	dist_matrix = zeros([N,N])
	for i in range(N):
		for j in range(i,N):
			s = difflib.SequenceMatcher(None,words[i],words[j])
			dist_matrix[i,j] = s.ratio()
	dist_matrix = 0.5*(dist_matrix+dist_matrix.T)
	return CustomKernel(dist_matrix)

print 'loading'
words = []
f = open("../../data/toy/words.dat")
for line in f:
	words.append(line[:-1])
f.close()
print 'loaded'

converter = KernelLocallyLinearEmbedding()
converter.set_k(10)
converter.set_target_dim(2)
converter.parallel.set_num_threads(1)
embedding = converter.embed_kernel(word_kernel(words[:200]))
embedding_matrix = embedding.get_feature_matrix()
fig = figure()
ax = fig.add_subplot(1,1,1)
ax.scatter(embedding_matrix[0,:],embedding_matrix[1,:],alpha=0.4,cmap=cm.Spectral,c=embedding_matrix[0,:]*embedding_matrix[1,:])

# hardcode ;)
words_to_show = ['finishing','publishing','standing',\
                 'shifted','insisted','tilted','blasted',\
                 'jumble','battle','gobble']

for i in xrange(0,200):
	if words[i] in words_to_show:
		ax.text(embedding_matrix[0,i]*1.1,1.25*embedding_matrix[1,i],words[i],fontsize=16,alpha=1.0)

axis('off')
show()




from modshogun import RealFeatures
from modshogun import MulticlassLabels
from modshogun import MCLDA
from pylab import pcolor, contour, colorbar, connect, show, plot, axis

import numpy as np
import util

N = 500
size = 100


# positive examples
mean_pos = [-1, 4]
cov_pos  = [[1,40], [50, -2]]

x_pos, y_pos = np.random.multivariate_normal(mean_pos, cov_pos, 500).T
plot(x_pos, y_pos, 'bo');

# negative examples
mean_neg = [0, -3]
cov_neg  = [[100,50], [20, 3]]

x_neg, y_neg = np.random.multivariate_normal(mean_neg, cov_neg, 500).T
plot(x_neg, y_neg, 'ro');

# train qda
labels = MulticlassLabels( np.concatenate([np.zeros(N), np.ones(N)]) )
pos = np.array([x_pos, y_pos])
neg = np.array([x_neg, y_neg])
features = RealFeatures( np.array(np.concatenate([pos, neg], 1)) )

lda = MCLDA()
lda.set_labels(labels)
lda.train(features)

# compute output plot iso-lines
xs = np.array(np.concatenate([x_pos, x_neg]))
ys = np.array(np.concatenate([y_pos, y_neg]))

x1_max = max(1.2*xs)
x1_min = min(1.2*xs)
x2_max = max(1.2*ys)
x2_min = min(1.2*ys)

x1 = np.linspace(x1_min, x1_max, size)
x2 = np.linspace(x2_min, x2_max, size)

x, y = np.meshgrid(x1, x2)

dense = RealFeatures( np.array((np.ravel(x), np.ravel(y))) )
dense_labels = lda.apply(dense).get_labels()

z = dense_labels.reshape((size, size))

pcolor(x, y, z, shading = 'interp')
contour(x, y, z, linewidths = 1, colors = 'black', hold = True)

axis([x1_min, x1_max, x2_min, x2_max])

connect('key_press_event', util.quit)

show()



#!/usr/bin/env python
from tools.load import LoadMatrix

lm=LoadMatrix()
data = lm.load_numbers('../data/fm_train_real.dat')

parameter_list = [[data]]

def preprocessor_pca_modular (data):
	from modshogun import RealFeatures
	from modshogun import PCA

	features = RealFeatures(data)

	preprocessor = PCA()
	preprocessor.init(features)
	preprocessor.apply_to_feature_matrix(features)

	return features


if __name__=='__main__':
	print('PCA')
	preprocessor_pca_modular(*parameter_list[0])




#!/usr/bin/env python
from tools.load import LoadMatrix
from numpy import double
lm=LoadMatrix()

traindat = double(lm.load_numbers('../data/fm_train_real.dat'))
testdat = double(lm.load_numbers('../data/fm_test_real.dat'))
traindna = lm.load_dna('../data/fm_train_dna.dat')
testdna = lm.load_dna('../data/fm_test_dna.dat')

parameter_list = [[traindat,testdat,traindna,testdna],[traindat,testdat,traindna,testdna]]
def kernel_combined_modular (fm_train_real=traindat,fm_test_real=testdat,fm_train_dna=traindna,fm_test_dna=testdna ):
	from modshogun import CombinedKernel, GaussianKernel, FixedDegreeStringKernel, LocalAlignmentStringKernel
	from modshogun import RealFeatures, StringCharFeatures, CombinedFeatures, DNA

	kernel=CombinedKernel()
	feats_train=CombinedFeatures()
	feats_test=CombinedFeatures()

	subkfeats_train=RealFeatures(fm_train_real)
	subkfeats_test=RealFeatures(fm_test_real)
	subkernel=GaussianKernel(10, 1.1)
	feats_train.append_feature_obj(subkfeats_train)
	feats_test.append_feature_obj(subkfeats_test)
	kernel.append_kernel(subkernel)

	subkfeats_train=StringCharFeatures(fm_train_dna, DNA)
	subkfeats_test=StringCharFeatures(fm_test_dna, DNA)
	degree=3
	subkernel=FixedDegreeStringKernel(10, degree)
	feats_train.append_feature_obj(subkfeats_train)
	feats_test.append_feature_obj(subkfeats_test)
	kernel.append_kernel(subkernel)

	subkfeats_train=StringCharFeatures(fm_train_dna, DNA)
	subkfeats_test=StringCharFeatures(fm_test_dna, DNA)
	subkernel=LocalAlignmentStringKernel(10)
	feats_train.append_feature_obj(subkfeats_train)
	feats_test.append_feature_obj(subkfeats_test)
	kernel.append_kernel(subkernel)

	kernel.init(feats_train, feats_train)
	km_train=kernel.get_kernel_matrix()
	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return km_train,km_test,kernel

if __name__=='__main__':
	print('Combined')
	kernel_combined_modular(*parameter_list[0])





#!/usr/bin/env python
from tools.load import LoadMatrix
lm=LoadMatrix()

traindat = lm.load_dna('../data/fm_train_dna.dat')
testdat = lm.load_dna('../data/fm_test_dna.dat')

parameter_list = [[traindat,testdat,20],[traindat,testdat,22]]
def kernel_weighted_degree_position_string_modular (fm_train_dna=traindat,fm_test_dna=testdat,degree=20):
	from modshogun import StringCharFeatures, DNA
	from modshogun import WeightedDegreePositionStringKernel, MSG_DEBUG

	feats_train=StringCharFeatures(fm_train_dna, DNA)
	#feats_train.io.set_loglevel(MSG_DEBUG)
	feats_test=StringCharFeatures(fm_test_dna, DNA)

	kernel=WeightedDegreePositionStringKernel(feats_train, feats_train, degree)

	from numpy import zeros,ones,float64,int32
	kernel.set_shifts(10*ones(len(fm_train_dna[0]), dtype=int32))
	kernel.set_position_weights(ones(len(fm_train_dna[0]), dtype=float64))

	km_train=kernel.get_kernel_matrix()
	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return km_train,km_test,kernel

if __name__=='__main__':
	print('WeightedDegreePositionString')
	kernel_weighted_degree_position_string_modular(*parameter_list[0])



"""Generator for Clustering
"""

from numpy import matrix
from shogun.Distance import EuclideanDistance
import shogun.Clustering as clustering

from shogun.Library import Math_init_random

import fileop
import featop
import dataop
import category


def _run (name, first_arg):
	"""
	Run generator for a specific clustering method.

	@param name Name of the clustering method to run.
	@param first_arg First argument to the clustering's constructor; so far, only this distinguishes the instantion of the different methods.
	"""

	# put some constantness into randomness
	Math_init_random(dataop.INIT_RANDOM)

	num_clouds=3
	params={
		'name': 'EuclideanDistance',
		'data': dataop.get_clouds(num_clouds, 5),
		'feature_class': 'simple',
		'feature_type': 'Real'
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'], params['data'])
	dfun=eval(params['name'])
	distance=dfun(feats['train'], feats['train'])
	output=fileop.get_output(category.DISTANCE, params)

	params={
		'name': name,
		'accuracy': 1e-8,
		first_arg: num_clouds
	}
	fun=eval('clustering.'+name)
	clustering=fun(params[first_arg], distance)
	clustering.train()

	if name=='KMeans':
		params['radi']=clustering.get_radiuses()
		params['centers']=clustering.get_cluster_centers()
	elif name=='Hierarchical':
		params['merge_distance']=clustering.get_merge_distances()
		params['pairs']=clustering.get_cluster_pairs()

	output.update(fileop.get_output(category.CLUSTERING, params))
	fileop.write(category.CLUSTERING, output)


def run ():
	"""Run all clustering methods."""

	_run('KMeans', 'k')
	_run('Hierarchical', 'merges')



"""Generator for Classifier"""

import numpy
import shogun.Library as library
import shogun.Classifier as classifier
from shogun.Kernel import *
from shogun.Distance import EuclidianDistance
from shogun.Features import Labels, RAWDNA

import fileop
import featop
import dataop
import category


##########################################################################
# svm
##########################################################################

def _get_svm (params, labels, feats, kernel):
	"""Return an SVM object.

	This function instantiates an SVM depending on the parameters.

	@param name Name of the SVM to instantiate
	@param labels Labels to be used for the SVM (if at all!)
	@param params Misc parameters for the SVM's constructor
	@return An SVM object
	"""

	try:
		svm=eval('classifier.'+params['name'])
	except AttributeError, e:
		return False

	if params['type']=='kernel':
		kernel.parallel.set_num_threads(params['num_threads'])
		kernel.init(feats['train'], feats['train'])

		if labels is None:
			return svm(params['C'], kernel)
		else:
			return svm(params['C'], kernel, labels)
	elif params['type']=='wdsvmocas':
		return svm(params['C'], params['degree'], params['degree'],
			feats['train'], labels)
	else:
		return svm(params['C'], feats['train'], labels)


def _get_svm_sum_alpha_and_sv (svm, ltype):
	"""Return sums of alphas and support vectors for given (MultiClass) SVM.
		Since alphas and support vectors are only used for comparison, it is
		alright to only regard their sums. Especially in case of
		MultiClassSVMs it makes test files smaller and easier to comprehend.

	@param svm (MultiClass) SVM to retrieve alphas from
	@param ltype label type of SVM
	@return 2-element list with sums of alphas and support vectors
	"""

	a=0
	sv=0
	if ltype=='series':
		for i in xrange(svm.get_num_svms()):
			subsvm=svm.get_svm(i)
			for item in subsvm.get_alphas().tolist():
				a+=item
			for item in subsvm.get_support_vectors().tolist():
				sv+=item
	else:
		for item in svm.get_alphas().tolist():
			a+=item
		for item in svm.get_support_vectors().tolist():
			sv+=item

	return a, sv


def _compute_svm (params, labels, feats, kernel, pout):
	"""Perform computations on SVM.

	Perform all necessary computations on SVM and gather the output.

	@param params misc parameters for the SVM's constructor
	@param labels labels to be used for the SVM (if at all)
	@param feats features to the SVM
	@param kernel kernel for kernel-SVMs
	@param pout previously gathered output data ready to be written to file
	"""

	svm=_get_svm(params, labels, feats, kernel)
	if not svm:
		return

	svm.parallel.set_num_threads(params['num_threads'])
	try:
		svm.set_epsilon(params['epsilon'])
	except AttributeError: #SGD does not have an accuracy parameter
		pass

	if params.has_key('bias_enabled'):
		svm.set_bias_enabled(params['bias_enabled'])
	if params.has_key('max_train_time'):
		svm.set_max_train_time(params['max_train_time'])
		params['max_train_time']=params['max_train_time']
	if params.has_key('linadd_enabled'):
		svm.set_linadd_enabled(params['linadd_enabled'])
	if params.has_key('batch_enabled'):
		svm.set_batch_computation_enabled(params['batch_enabled'])

	svm.train()

	if ((params.has_key('bias_enabled') and params['bias_enabled']) or
		params['type']=='kernel'):
		params['bias']=svm.get_bias()

	if params['type']=='kernel':
		params['alpha_sum'], params['sv_sum']= \
			_get_svm_sum_alpha_and_sv(svm, params['label_type'])
		kernel.init(feats['train'], feats['test'])
	elif params['type']=='linear' or params['type']=='wdsvmocas':
		svm.set_features(feats['test'])

	params['classified']=svm.apply().get_labels()

	output=fileop.get_output(category.CLASSIFIER, params)
	if pout:
		output.update(pout)
	fileop.write(category.CLASSIFIER, output)


def _loop_svm (svms, params, feats, kernel=None, pout=None):
	"""Loop through SVM computations, only slightly differing in parameters.

	Loop through SVM computations with little variations in the parameters for
	the SVM. Not necessarily used by all SVMs in this generator.

	@param svms tuple of SVM names to loop through
	@param params parameters to the SVMs
	@param feats features to the SVMs
	@param kernel kernel for kernel-based SVMs
	@param pout previously gathered output data ready to be written to file
	"""

	for name in svms:
		parms={ 'name': name, 'num_threads': 1, 'C': .017, 'epsilon': 1e-5 }
		parms['accuracy']=parms['epsilon']*10
		parms.update(params)

		if params['label_type'] is not None:
			parms['labels'], labels=dataop.get_labels(
				feats['train'].get_num_vectors(), params['label_type'])
		else:
			labels=None

		_compute_svm(parms, labels, feats, kernel, pout)
		parms['C']=.23
		_compute_svm(parms, labels, feats, kernel, pout)
		parms['C']=1.5
		_compute_svm(parms, labels, feats, kernel, pout)
		parms['C']=30
		_compute_svm(parms, labels, feats, kernel, pout)

		if params['type']=='kernel':
			_compute_svm(parms, labels, feats, kernel, pout)

		parms['num_threads']=16
		_compute_svm(parms, labels, feats, kernel, pout)


def _run_svm_kernel ():
	"""Run all kernel-based SVMs."""

	kparams={
		'name': 'Gaussian',
		'args': {'key': ('width',), 'val': (1.5,)},
		'feature_class': 'simple',
		'feature_type': 'Real',
		'data': dataop.get_clouds(2)
	}
	feats=featop.get_features(
		kparams['feature_class'], kparams['feature_type'], kparams['data'])
	kernel=GaussianKernel(10, *kparams['args']['val'])
	output=fileop.get_output(category.KERNEL, kparams)

	svms=('SVMLight', 'LibSVM', 'GPBTSVM', 'MPDSVM')
	params={
		'type': 'kernel',
		'label_type': 'twoclass'
	}
	_loop_svm(svms, params, feats, kernel, output)

	svms=('LibSVMOneClass',)
	params['label_type']=None
	_loop_svm(svms, params, feats, kernel, output)

	svms=('LibSVMMultiClass', 'GMNPSVM')
	params['label_type']='series'
	kparams['data']=dataop.get_clouds(3)
	feats=featop.get_features(
		kparams['feature_class'], kparams['feature_type'], kparams['data'])
	output=fileop.get_output(category.KERNEL, kparams)
	_loop_svm(svms, params, feats, kernel, output)

	svms=('SVMLight', 'GPBTSVM')
	params['label_type']='twoclass'
	kparams={
		'name': 'Linear',
		'feature_class': 'simple',
		'feature_type': 'Real',
		'data': dataop.get_clouds(2),
		'normalizer': AvgDiagKernelNormalizer()
	}
	feats=featop.get_features(
		kparams['feature_class'], kparams['feature_type'], kparams['data'])
	kernel=LinearKernel()
	kernel.set_normalizer(kparams['normalizer'])
	output=fileop.get_output(category.KERNEL, kparams)
	_loop_svm(svms, params, feats, kernel, output)

	kparams={
		'name': 'CommWordString',
		'args': {'key': ('use_sign',), 'val': (False,)},
		'data': dataop.get_dna(),
		'feature_class': 'string_complex',
		'feature_type': 'Word'
	}
	feats=featop.get_features(
		kparams['feature_class'], kparams['feature_type'], kparams['data'])
	kernel=CommWordStringKernel(10, *kparams['args']['val'])
	output=fileop.get_output(category.KERNEL, kparams)
	_loop_svm(svms, params, feats, kernel, output)

	kparams={
		'name': 'CommUlongString',
		'args': {'key': ('use_sign',), 'val': (False,)},
		'data': dataop.get_dna(),
		'feature_class': 'string_complex',
		'feature_type': 'Ulong'
	}
	feats=featop.get_features(
		kparams['feature_class'], kparams['feature_type'], kparams['data'])
	kernel=CommUlongStringKernel(10, *kparams['args']['val'])
	output=fileop.get_output(category.KERNEL, kparams)
	_loop_svm(svms, params, feats, kernel, output)

	kparams={
		'name': 'WeightedDegreeString',
		'args': {'key': ('degree',), 'val': (3,)},
		'data': dataop.get_dna(),
		'feature_class': 'string',
		'feature_type': 'Char'
	}
	feats=featop.get_features(
		kparams['feature_class'], kparams['feature_type'], kparams['data'])
	kernel=WeightedDegreeStringKernel(*kparams['args']['val'])
	output=fileop.get_output(category.KERNEL, kparams)
	_loop_svm(svms, params, feats, kernel, output)
	params['linadd_enabled']=True
	_loop_svm(svms, params, feats, kernel, output)
	params['batch_enabled']=True
	_loop_svm(svms, params, feats, kernel, output)

	kparams={
		'name': 'WeightedDegreePositionString',
		'args': {'key': ('degree',), 'val': (20,)},
		'data': dataop.get_dna(),
		'feature_class': 'string',
		'feature_type': 'Char'
	}
	feats=featop.get_features(
		kparams['feature_class'], kparams['feature_type'], kparams['data'])
	kernel=WeightedDegreePositionStringKernel(10, *kparams['args']['val'])
	output=fileop.get_output(category.KERNEL, kparams)
	del params['linadd_enabled']
	del params['batch_enabled']
	_loop_svm(svms, params, feats, kernel, output)
	params['linadd_enabled']=True
	_loop_svm(svms, params, feats, kernel, output)
	params['batch_enabled']=True
	_loop_svm(svms, params, feats, kernel, output)


def _run_svm_linear ():
	"""Run all SVMs based on (Sparse) Linear Classifiers."""

	params={
		'type': 'linear',
		'bias_enabled': False,
		'data': dataop.get_clouds(2),
		'feature_class': 'simple',
		'feature_type': 'Real',
		'label_type': 'twoclass'
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'],
		params['data'], sparse=True)

	svms=('LibLinear', 'SVMLin', 'SVMSGD')
	params['bias_enabled']=True
	_loop_svm(svms, params, feats)

	# SubGradientSVM needs max_train_time to terminate
	svms=('SubGradientSVM',)
	params['bias_enabled']=False
	params['max_train_time']=.5 # up to 2. does not improve test results :(
	_loop_svm(svms, params, feats)

	svms=('SVMOcas',)
	_loop_svm(svms, params, feats)

	params={
		'type': 'linear',
		'bias_enabled': False,
		'label_type': 'twoclass',
		'feature_class': 'wd',
		'feature_type': 'Byte',
		'data': dataop.get_dna(),
		'alphabet': 'RAWDNA',
		'order': 1
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'],
		params['data'], params['order'])
	_loop_svm(svms, params, feats)



##########################################################################
# other classifiers
##########################################################################

def _run_perceptron ():
	"""Run Perceptron classifier."""

	params={
		'name': 'Perceptron',
		'type': 'perceptron',
		'num_threads': 1,
		'learn_rate': .1,
		'max_iter': 1000,
		'data': dataop.get_clouds(2),
		'feature_class': 'simple',
		'feature_type': 'Real',
		'label_type': 'twoclass',
		'accuracy': 1e-7
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'], params['data'])
	num_vec=feats['train'].get_num_vectors()
	params['labels'], labels=dataop.get_labels(num_vec, params['label_type'])

	perceptron=classifier.Perceptron(feats['train'], labels)
	perceptron.parallel.set_num_threads(params['num_threads'])
	perceptron.set_learn_rate(params['learn_rate'])
	perceptron.set_max_iter(params['max_iter'])
	perceptron.train()

	params['bias']=perceptron.get_bias()
	perceptron.set_features(feats['test'])
	params['classified']=perceptron.classify().get_labels()

	output=fileop.get_output(category.CLASSIFIER, params)
	fileop.write(category.CLASSIFIER, output)


def _run_knn ():
	"""Run K-Nearest-Neighbour classifier.
	"""

	params={
		'name': 'EuclidianDistance',
		'data': dataop.get_clouds(2),
		'feature_class': 'simple',
		'feature_type': 'Real'
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'], params['data'])
	dfun=eval(params['name'])
	distance=dfun(feats['train'], feats['train'])
	output=fileop.get_output(category.DISTANCE, params)

	params={
		'name': 'KNN',
		'type': 'knn',
		'num_threads': 1,
		'k': 3,
		'label_type': 'twoclass',
		'accuracy': 1e-8
	}
	params['labels'], labels=dataop.get_labels(
		feats['train'].get_num_vectors(), params['label_type'])

	knn=classifier.KNN(params['k'], distance, labels)
	knn.parallel.set_num_threads(params['num_threads'])
	knn.train()

	distance.init(feats['train'], feats['test'])
	params['classified']=knn.classify().get_labels()

	output.update(fileop.get_output(category.CLASSIFIER, params))
	fileop.write(category.CLASSIFIER, output)


def _run_lda ():
	"""Run Linear Discriminant Analysis classifier."""

	params={
		'name': 'LDA',
		'type': 'lda',
		'gamma': 0.1,
		'num_threads': 1,
		'data': dataop.get_clouds(2),
		'feature_class': 'simple',
		'feature_type': 'Real',
		'label_type': 'twoclass',
		'accuracy': 1e-7
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'], params['data'])
	params['labels'], labels=dataop.get_labels(
		feats['train'].get_num_vectors(), params['label_type'])

	lda=classifier.LDA(params['gamma'], feats['train'], labels)
	lda.parallel.set_num_threads(params['num_threads'])
	lda.train()

	lda.set_features(feats['test'])
	params['classified']=lda.classify().get_labels()

	output=fileop.get_output(category.CLASSIFIER, params)
	fileop.write(category.CLASSIFIER, output)


def _run_wdsvmocas ():
	"""Run Weighted Degree SVM Ocas classifier."""

	svms=('WDSVMOcas',)
	params={
		'type': 'wdsvmocas',
		'degree': 1,
		'bias_enabled': False,
		#'data': dataop.get_rawdna(),
		'data': dataop.get_dna(
			dataop.NUM_VEC_TRAIN, dataop.NUM_VEC_TRAIN, dataop.NUM_VEC_TRAIN),
		'feature_class': 'string_complex',
		'feature_type': 'Byte',
		'alphabet': 'RAWDNA',
		'label_type': 'twoclass',
		'order': 1,
		'gap': 0,
		'reverse': False
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'],
		params['data'], eval(params['alphabet']),
		params['order'], params['gap'], params['reverse'])
	_loop_svm(svms, params, feats)


##########################################################################
# public
##########################################################################

def run ():
	"""Run generator for all classifiers."""

#	_run_svm_kernel()
	_run_svm_linear()
#	_run_knn()
#	_run_lda()
#	_run_perceptron()
#	_run_wdsvmocas()




"""
Test Classifier
"""

from numpy import double
from sg import sg

import util


def _set_classifier (indata, prefix):
	if indata.has_key(prefix+'labels'):
		sg('set_labels', 'TRAIN', double(indata[prefix+'labels']))

	cname=util.fix_classifier_name_inconsistency(indata[prefix+'name'])
	sg('new_classifier', cname)

	if indata.has_key(prefix+'bias'):
		sg('svm_use_bias', True)
	else:
		sg('svm_use_bias', False)

	if indata.has_key(prefix+'epsilon'):
		sg('svm_epsilon', indata[prefix+'epsilon'])
	if indata.has_key(prefix+'max_train_time'):
		sg('svm_max_train_time', indata[prefix+'max_train_time'])
	if indata.has_key(prefix+'linadd_enabled'):
		sg('use_linadd', True)
	if indata.has_key(prefix+'batch_enabled'):
		sg('use_batch_computation', True)
	if indata.has_key(prefix+'num_threads'):
		sg('threads', indata[prefix+'num_threads'])


def _train (indata, prefix):
	if indata[prefix+'type']=='knn':
		sg('train_classifier', indata[prefix+'k'])
	elif indata[prefix+'type']=='lda':
		sg('train_classifier', indata[prefix+'gamma'])
	elif indata[prefix+'type']=='perceptron':
		# does not converge
		try:
			sg('train_classifier')
		except RuntimeError:
			import sys
			sys.exit(0)
	else:
		if indata.has_key(prefix+'C'):
			sg('c', double(indata[prefix+'C']))
		sg('train_classifier')


def _get_alpha_and_sv(indata, prefix):
	if not indata.has_key(prefix+'alpha_sum') and \
		not indata.has_key(prefix+'sv_sum'):
		return None, None

	a=0
	sv=0
	if indata.has_key(prefix+'label_type') and \
		indata[prefix+'label_type']=='series':
		for i in xrange(sg('get_num_svms')):
			[dump, weights]=sg('get_svm', i)
			weights=weights.T
			for item in weights[0].tolist():
				a+=item
			for item in weights[1].tolist():
				sv+=item
		a=abs(a-indata[prefix+'alpha_sum'])
		sv=abs(sv-indata[prefix+'sv_sum'])
	else:
		[dump, weights]=sg('get_svm')
		weights=weights.T
		for item in weights[0].tolist():
			a+=item
		a=abs(a-indata[prefix+'alpha_sum'])
		for item in weights[1].tolist():
			sv+=item
		sv=abs(sv-indata[prefix+'sv_sum'])

	return a, sv


def _evaluate (indata, prefix):
	alphas=0
	bias=0
	sv=0

	if indata[prefix+'type']=='lda':
		pass
	else:
		if indata.has_key(prefix+'label_type') and \
			indata[prefix+'label_type'] != 'series' and \
			indata.has_key(prefix+'bias'):
			[b, weights]=sg('get_svm')
			weights=weights.T
			bias=abs(b-indata[prefix+'bias'])

		alphas, sv=_get_alpha_and_sv(indata, prefix)

	classified=max(abs(sg('classify')-indata[prefix+'classified']))

	return util.check_accuracy(indata[prefix+'accuracy'],
		alphas=alphas, bias=bias, support_vectors=sv, classified=classified)

########################################################################
# public
########################################################################

def test (indata):
	prefix='classifier_'

	if indata[prefix+'type']=='kernel':
		feature_prefix='kernel_'
	elif indata[prefix+'type']=='knn':
		feature_prefix='distance_'
	else:
		feature_prefix='classifier_'

	try:
		util.set_features(indata, feature_prefix)
	except NotImplementedError, e:
		print e
		return True

	if indata[prefix+'type']=='kernel':
		util.set_and_train_kernel(indata)
	elif indata[prefix+'type']=='knn':
		util.set_and_train_distance(indata)

	try:
		_set_classifier(indata, prefix)
	except RuntimeError:
		print "%s is disabled/unavailable!" % indata[prefix+'name']
		return True

	_train(indata, prefix)

	return _evaluate(indata, prefix)




#!/usr/bin/env python
parameter_list=[[10,7,0,0]]

def tests_check_commwordkernel_memleak_modular (num, order, gap, reverse):
	import gc
	from modshogun import Alphabet,StringCharFeatures,StringWordFeatures,DNA
	from modshogun import SortWordString, MSG_DEBUG
	from modshogun import CommWordStringKernel, IdentityKernelNormalizer
	from numpy import mat

	POS=[num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'TTGT', num*'TTGT', num*'TTGT',num*'TTGT', num*'TTGT',
	num*'TTGT',num*'TTGT', num*'TTGT', num*'TTGT',num*'TTGT', num*'TTGT',
	num*'TTGT',num*'TTGT', num*'TTGT', num*'TTGT',num*'TTGT', num*'TTGT',
	num*'TTGT',num*'TTGT', num*'TTGT', num*'TTGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT']
	NEG=[num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'TTGT', num*'TTGT', num*'TTGT',num*'TTGT', num*'TTGT',
	num*'TTGT',num*'TTGT', num*'TTGT', num*'TTGT',num*'TTGT', num*'TTGT',
	num*'TTGT',num*'TTGT', num*'TTGT', num*'TTGT',num*'TTGT', num*'TTGT',
	num*'TTGT',num*'TTGT', num*'TTGT', num*'TTGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT',num*'ACGT', num*'ACGT',
	num*'ACGT',num*'ACGT', num*'ACGT', num*'ACGT']

	for i in range(10):
		alpha=Alphabet(DNA)
		traindat=StringCharFeatures(alpha)
		traindat.set_features(POS+NEG)
		trainudat=StringWordFeatures(traindat.get_alphabet());
		trainudat.obtain_from_char(traindat, order-1, order, gap, reverse)
		#trainudat.io.set_loglevel(MSG_DEBUG)
		pre = SortWordString()
		#pre.io.set_loglevel(MSG_DEBUG)
		pre.init(trainudat)
		trainudat.add_preprocessor(pre)
		trainudat.apply_preprocessor()
		spec = CommWordStringKernel(10, False)
		spec.set_normalizer(IdentityKernelNormalizer())
		spec.init(trainudat, trainudat)
		K=spec.get_kernel_matrix()

	del POS
	del NEG
	del order
	del gap
	del reverse
	return K

if __name__=='__main__':
	print('Leak Check Comm Word Kernel')
	tests_check_commwordkernel_memleak_modular(*parameter_list[0])



#!/usr/bin/env python
from tools.load import LoadMatrix

lm=LoadMatrix()
data = lm.load_numbers('../data/fm_train_real.dat')

parameter_list = [[data,0.0],[data,1.0]]

def mathematics_sparseinversecovariance_modular (data,lc):
	from modshogun import SparseInverseCovariance
	from numpy import dot

	sic = SparseInverseCovariance()
	S = dot(data,data.T)
	Si = sic.estimate(S,lc)

	return Si


if __name__=='__main__':
	print('SparseInverseCovariance')
	mathematics_sparseinversecovariance_modular(*parameter_list[0])





#!/usr/bin/env python
traindat = '../data/fm_train_byte.dat'
testdat = '../data/fm_test_byte.dat'

parameter_list=[[traindat,testdat],[traindat,testdat]]

def kernel_linear_byte_modular (train_fname=traindat,test_fname=testdat):
	from modshogun import LinearKernel, ByteFeatures, CSVFile

	feats_train=ByteFeatures(CSVFile(train_fname))
	feats_test=ByteFeatures(CSVFile(test_fname))

	kernel=LinearKernel(feats_train, feats_train)
	km_train=kernel.get_kernel_matrix()

	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return kernel

if __name__=='__main__':
	print('LinearByte')
	kernel_linear_byte_modular(*parameter_list[0])



""" Utilities for matplotlib examples """

import pylab
from numpy import ones, array, meshgrid, linspace, concatenate, ravel, min, max
from numpy.random import randn

QUITKEY='q'
NUM_EXAMPLES=200
DISTANCE=2

def quit (event):
	if event.key==QUITKEY or event.key==QUITKEY.upper():
		pylab.close()

def set_title (title):
	quitmsg=" (press '"+QUITKEY+"' to quit)"
	complete=title+quitmsg
	manager=pylab.get_current_fig_manager()

	# now we have to wrap the toolkit
	if hasattr(manager, 'window'):
		if hasattr(manager.window, 'setCaption'): # QT
			manager.window.setCaption(complete)
		if hasattr(manager.window, 'set_title'): # GTK
			manager.window.set_title(complete)
		elif hasattr(manager.window, 'title'): # TK
			manager.window.title(complete)


def get_traindata():
	return concatenate(
		(randn(2, NUM_EXAMPLES)+DISTANCE, randn(2, NUM_EXAMPLES)-DISTANCE),
		axis=1)


def get_meshgrid(traindata):
	x1=linspace(1.2*min(traindata), 1.2*max(traindata), 50)
	x2=linspace(1.2*min(traindata), 1.2*max(traindata), 50)
	return meshgrid(x1,x2)


def get_testdata(x, y):
	return array((ravel(x), ravel(y)))


def get_labels(raw=False):
	return concatenate(
		(-ones([1, NUM_EXAMPLES]), ones([1, NUM_EXAMPLES])),
		axis=1)[0]




#!/usr/bin/env python
from modshogun import *
from numpy import array
import os

parameter_list=[[[[1.0,2,3],[4,5,6]]]]

def serialization_matrix_modular (m):
	feats=RealFeatures(array(m))
	#feats.io.set_loglevel(0)
	fstream = SerializableAsciiFile("foo.asc", "w")
	feats.save_serializable(fstream)

	l=MulticlassLabels(array([1.0,2,3]))
	fstream = SerializableAsciiFile("foo2.asc", "w")
	l.save_serializable(fstream)

	os.unlink("foo.asc")
	os.unlink("foo2.asc")

if __name__=='__main__':
	print('Serialization Matrix Modular')
	serialization_matrix_modular(*parameter_list[0])



#!/usr/bin/env python
from numpy import *
from modshogun import *

x=array([[20.0,15,15],[10,20,20]])
y=array([[21.0,21,18],[19,19,22]])
z=array([[15.0,27,18],[32,5,23]])


parameter_list = [[x,concatenate((x,y,z),1)]]

def library_fisher2x3_modular (table, tables):
	pval=Statistics_fishers_exact_test_for_2x3_table(table)
	pvals=Statistics_fishers_exact_test_for_multiple_2x3_tables(tables)
	return (pval,pvals)

if __name__=='__main__':
	print('Fisher 2x3')
	library_fisher2x3_modular(*parameter_list[0])



from pylab import figure,show,connect,hist,plot,legend
from numpy import array, append, arange, empty, exp
from modshogun import Gaussian, GMM
from modshogun import RealFeatures
import util

util.set_title('SMEM for 1d GMM example')

#set the parameters
max_iter=100
max_cand=5
min_cov=1e-9
max_em_iter=1000
min_change=1e-9

#setup the real GMM
real_gmm=GMM(3)

real_gmm.set_nth_mean(array([-2.0]), 0)
real_gmm.set_nth_mean(array([0.0]), 1)
real_gmm.set_nth_mean(array([2.0]), 2)

real_gmm.set_nth_cov(array([[0.3]]), 0)
real_gmm.set_nth_cov(array([[0.1]]), 1)
real_gmm.set_nth_cov(array([[0.2]]), 2)

real_gmm.set_coef(array([0.3, 0.5, 0.2]))

#generate training set from real GMM
generated=array([real_gmm.sample()])
for i in range(199):
    generated=append(generated, array([real_gmm.sample()]), axis=1)

feat_train=RealFeatures(generated)

#train GMM using SMEM and print log-likelihood
est_smem_gmm=GMM(3)
est_smem_gmm.train(feat_train)
print est_smem_gmm.train_smem(max_iter, max_cand, min_cov, max_em_iter, min_change)
#train GMM using EM and print log-likelihood
est_em_gmm=GMM(3)
est_em_gmm.train(feat_train)
print est_em_gmm.train_em(min_cov, max_em_iter, min_change)

#plot real GMM, data and both estimated GMMs
min_gen=min(min(generated))
max_gen=max(max(generated))
plot_real=empty(0)
plot_est_smem=empty(0)
plot_est_em=empty(0)
for i in arange(min_gen, max_gen, 0.001):
    plot_real=append(plot_real, array([exp(real_gmm.cluster(array([i]))[3])]))
    plot_est_smem=append(plot_est_smem, array([exp(est_smem_gmm.cluster(array([i]))[3])]))
    plot_est_em=append(plot_est_em, array([exp(est_em_gmm.cluster(array([i]))[3])]))
real_plot=plot(arange(min_gen, max_gen, 0.001), plot_real, "b")
est_em_plot=plot(arange(min_gen, max_gen, 0.001), plot_est_em, "g")
est_smem_plot=plot(arange(min_gen, max_gen, 0.001), plot_est_smem, "r")
real_hist=hist(generated.transpose(), bins=50, normed=True, fc="gray")
legend(("Real GMM", "Estimated EM GMM", "Estimated SMEM GMM"))
connect('key_press_event', util.quit)
show()



import os
import sys

insertions=0
deletions=0
files=0
FROMVER=""
if len(sys.argv)>1:
	FROMVER=sys.argv[1]
TOVER=""
if len(sys.argv)>2:
	TOVER=sys.argv[2]
TMPNAME=os.tmpnam()
VER=""
if len(FROMVER)>0:
	VER=FROMVER+'..'
if len(TOVER)>0:
	if len(VER)==0:
		VER='..'
	VER=VER+TOVER

os.system('git log --oneline --shortstat %s >%s' % (VER,TMPNAME))
for line in file(TMPNAME).readlines():
	if line.find('file') == -1:
		continue
	if line.find('changed') == -1:
		continue
	if line.find('insertion') == -1 and line.find('deletion') == -1:
		continue

	entries=line.split(',')
	for e in entries:
		if e.find('file') != -1:
			files+=int(e.strip().split(' ')[0])
		elif e.find('insertion') != -1:
			insertions+=int(e.strip().split(' ')[0])
		elif e.find('deletion') != -1:
			deletions+=int(e.strip().split(' ')[0])

print "Files changed: %d" % files
print "Insertions: %d" % insertions
print "Deletions: %d" % deletions

os.unlink(TMPNAME)



#!/usr/bin/env python

import numpy as np
import matplotlib.pyplot as plt

from modshogun	import RealFeatures
from modshogun	import MulticlassModel, MulticlassSOLabels, RealNumber, DualLibQPBMSOSVM
from modshogun	import BMRM, PPBMRM, P3BMRM
from modshogun	import StructuredAccuracy

def fill_data(cnt, minv, maxv):
	x1 = np.linspace(minv, maxv, cnt)
	a, b = np.meshgrid(x1, x1)
	X = np.array((np.ravel(a), np.ravel(b)))
	y = np.zeros((1, cnt*cnt))
	tmp = cnt*cnt;
	y[0, tmp/3:(tmp/3)*2]=1
	y[0, tmp/3*2:(tmp/3)*3]=2
	return X, y.flatten()

def gen_data():
	covs = np.array([[[0., -1. ], [2.5,  .7]],
		[[3., -1.5], [1.2, .3]],
		[[ 2,  0  ], [ .0,  1.5 ]]])
	X = np.r_[np.dot(np.random.randn(N, dim), covs[0]) + np.array([0, 10]),
			np.dot(np.random.randn(N, dim), covs[1]) + np.array([-10, -10]),
			np.dot(np.random.randn(N, dim), covs[2]) + np.array([10, -10])];
	Y = np.hstack((np.zeros(N), np.ones(N), 2*np.ones(N)))
	return X, Y

def get_so_labels(out):
	N = out.get_num_labels()
	l = np.zeros(N)
	for i in xrange(N):
		l[i] = RealNumber.obtain_from_generic(out.get_label(i)).value
	return l

# Number of classes
M = 3
# Number of samples of each class
N = 1000
# Dimension of the data
dim = 2

X, y = gen_data()

cnt = 250

X2, y2 = fill_data(cnt, np.min(X), np.max(X))

labels = MulticlassSOLabels(y)
features = RealFeatures(X.T)

model = MulticlassModel(features, labels)

lambda_ = 1e1
sosvm = DualLibQPBMSOSVM(model, labels, lambda_)

sosvm.set_cleanAfter(10)	# number of iterations that cutting plane has to be inactive for to be removed
sosvm.set_cleanICP(True)	# enables inactive cutting plane removal feature
sosvm.set_TolRel(0.001)		# set relative tolerance
sosvm.set_verbose(True)		# enables verbosity of the solver
sosvm.set_cp_models(16)		# set number of cutting plane models
sosvm.set_solver(BMRM)		# select training algorithm
#sosvm.set_solver(PPBMRM)
#sosvm.set_solver(P3BMRM)

sosvm.train()

res = sosvm.get_result()
Fps = np.array(res.get_hist_Fp_vector())
Fds = np.array(res.get_hist_Fp_vector())
wdists = np.array(res.get_hist_wdist_vector())

plt.figure()
plt.subplot(221)
plt.title('Fp and Fd history')
plt.plot(xrange(res.get_n_iters()), Fps, hold=True)
plt.plot(xrange(res.get_n_iters()), Fds, hold=True)
plt.subplot(222)
plt.title('w dist history')
plt.plot(xrange(res.get_n_iters()), wdists)

# Evaluation
out = sosvm.apply()

Evaluation = StructuredAccuracy()
acc = Evaluation.evaluate(out, labels)

print "Correct classification rate: %0.4f%%" % ( 100.0*acc )

# show figure
Z = get_so_labels(sosvm.apply(RealFeatures(X2)))
x = (X2[0,:]).reshape(cnt, cnt)
y = (X2[1,:]).reshape(cnt, cnt)
z = Z.reshape(cnt, cnt)

plt.subplot(223)
plt.pcolor(x, y, z, shading='interp')
plt.contour(x, y, z, linewidths=1, colors='black', hold=True)
plt.plot(X[:,0], X[:,1], 'yo')
plt.axis('tight')
plt.title('Classification')
plt.show()



#!/usr/bin/env python
from tools.load import LoadMatrix
lm=LoadMatrix()
traindat = lm.load_dna('../data/fm_train_dna.dat')
testdat = lm.load_dna('../data/fm_test_dna.dat')

parameter_list = [[traindat,testdat,3,1.2,10],[traindat,testdat,4,1.3,10]]

def kernel_oligo_string_modular (fm_train_dna=traindat,fm_test_dna=testdat,k=3,width=1.2,size_cache=10):
	from modshogun import StringCharFeatures, DNA
	from modshogun import OligoStringKernel

	feats_train=StringCharFeatures(fm_train_dna, DNA)
	feats_test=StringCharFeatures(fm_test_dna, DNA)

	kernel=OligoStringKernel(size_cache, k, width)
	kernel.init(feats_train, feats_train)

	km_train=kernel.get_kernel_matrix()
	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return km_train,km_test,kernel

if __name__=='__main__':
	print('OligoString')
	kernel_oligo_string_modular(*parameter_list[0])



from sg import sg
from pylab import plot, show, connect
from numpy import array,transpose,sin,double

import util

util.set_title('SVR Regression')

sg('new_regression', 'LIBSVR')
features=array([range(0,100)],dtype=double)
features.resize(1,100)
labels=sin(features)[0]
sg('set_features', "TRAIN", features)
sg('set_labels', "TRAIN", labels)
sg('set_kernel', 'GAUSSIAN', 'REAL', 20, 10.)
sg('c', 1.)
sg('train_regression')
[bias, alphas]=sg('get_svm');
sg('set_features', "TEST", features)
out=sg('classify');

plot(features[0],labels,'b-')
plot(features[0],labels,'bo')
plot(features[0],out,'r-')
plot(features[0],out,'ro')
connect('key_press_event', util.quit)
show()



"""
Test Distribution
"""
from numpy import inf, nan
from modshogun import *

import util

def _evaluate (indata):
	prefix='distribution_'
	feats=util.get_features(indata, prefix)

	if indata[prefix+'name']=='HMM':
		distribution=HMM(feats['train'], indata[prefix+'N'],
			indata[prefix+'M'], indata[prefix+'pseudo'])
		distribution.train()
		distribution.baum_welch_viterbi_train(BW_NORMAL)
	else:
		dfun=eval(indata[prefix+'name'])
		distribution=dfun(feats['train'])
		distribution.train()

	likelihood=distribution.get_log_likelihood_sample()
	num_examples=feats['train'].get_num_vectors()
	num_param=distribution.get_num_model_parameters()
	derivatives=0
	for i in range(num_param):
		for j in range(num_examples):
			val=distribution.get_log_derivative(i, j)
			if val!=-inf and val!=nan: # only consider sparse matrix!
				derivatives+=val

	derivatives=abs(derivatives-indata[prefix+'derivatives'])
	likelihood=abs(likelihood-indata[prefix+'likelihood'])

	if indata[prefix+'name']=='HMM':
		best_path=0
		best_path_state=0
		for i in range(indata[prefix+'num_examples']):
			best_path+=distribution.best_path(i)
			for j in range(indata[prefix+'N']):
				best_path_state+=distribution.get_best_path_state(i, j)

		best_path=abs(best_path-indata[prefix+'best_path'])
		best_path_state=abs(best_path_state-\
			indata[prefix+'best_path_state'])

		return util.check_accuracy(indata[prefix+'accuracy'],
			derivatives=derivatives, likelihood=likelihood,
			best_path=best_path, best_path_state=best_path_state)
	else:
		return util.check_accuracy(indata[prefix+'accuracy'],
			derivatives=derivatives, likelihood=likelihood)


########################################################################
# public
########################################################################

def test (indata):
	return _evaluate(indata)




#!/usr/bin/env python
from tools.load import LoadMatrix
lm=LoadMatrix()

traindna = lm.load_dna('../data/fm_train_dna.dat')
testdna = lm.load_dna('../data/fm_test_dna.dat')


parameter_list = [[traindna,testdna,3,0,False],[traindna,testdna,3,0,False]]

def distance_canberraword_modular (fm_train_dna=traindna,fm_test_dna=testdna,order=3,gap=0,reverse=False):
	from modshogun import StringCharFeatures, StringWordFeatures, DNA
	from modshogun import SortWordString
	from modshogun import CanberraWordDistance

	charfeat=StringCharFeatures(DNA)
	charfeat.set_features(fm_train_dna)
	feats_train=StringWordFeatures(charfeat.get_alphabet())
	feats_train.obtain_from_char(charfeat, order-1, order, gap, reverse)
	preproc=SortWordString()
	preproc.init(feats_train)
	feats_train.add_preprocessor(preproc)
	feats_train.apply_preprocessor()

	charfeat=StringCharFeatures(DNA)
	charfeat.set_features(fm_test_dna)
	feats_test=StringWordFeatures(charfeat.get_alphabet())
	feats_test.obtain_from_char(charfeat, order-1, order, gap, reverse)
	feats_test.add_preprocessor(preproc)
	feats_test.apply_preprocessor()

	distance=CanberraWordDistance(feats_train, feats_train)

	dm_train=distance.get_distance_matrix()
	distance.init(feats_train, feats_test)
	dm_test=distance.get_distance_matrix()
	return distance,dm_train,dm_test

if __name__=='__main__':
	print('CanberraWordDistance')
	distance_canberraword_modular(*parameter_list[0])
